#include <windows.h>
#include "GL/glut.h"
#include <iostream>
#include <fstream>
#include <string>
#include "ltga.h"
#include "pixel.h"
#include "wrapper.h"
#include "menu.h"

////////////////////////////////////////////////////////////////////////////////////
void main(int argc,char **argv)
{
	std::string colorMap;
	std::string opacityMap;
	std::string velocityMap;
	std::string heightMap;
	std::string scaleMap;

	bool opacityFile = false;
	bool velocityFile = false;
	bool heightFile = false;
	bool scaleFile = false;

	std::string colorExtension;
	std::string opacityExtension;
	std::string velocityExtension;
	std::string heightExtension;
	std::string scaleExtension;

	colorExtension = (".pColor");
	opacityExtension = (".pOpacity");
	velocityExtension = (".pVelocity");
	heightExtension = (".pHeight");
	scaleExtension = (".pScale");

	float * heightArray;
	unsigned int nPixels;

	wrapper colorImage;
	wrapper opacityImage;
	wrapper velocityImage;
	wrapper heightImage;
	wrapper scaleImage;

	std::string ppcVelocityPath;
	std::ifstream ppcVelocity;
	std::ifstream ppcHeight;
	int checkPixelCount = 0;

    menu ppcMenu;
	
	//////////////////////////
	std::cout << "PPC-cruncher v1.0 beta1 provided by Stefan D'hondt" << std::endl;
	std::cout << "Please direct your fanmail to \"supersteffieke@yahoo.co.uk\"" << std::endl;
	std::cout << std::endl;
	//////////////////////////
	//display menu
	ppcMenu.showMainMenu();
	
	unsigned int tickCount = GetTickCount();
	
	//set image path in wrapper
	colorImage.setPath(ppcMenu.getpColorMap());
	opacityImage.setPath(ppcMenu.getpOpacityMap());
	velocityImage.setPath(ppcMenu.getpVelocityMap());
	heightImage.setPath(ppcMenu.getpHeightMap());
	scaleImage.setPath(ppcMenu.getpScaleMap());

	do
	{
		colorImage.setPath(ppcMenu.getpColorMap());

		if (colorImage.getPath().size() != 0)
		{
			//read in images
			if (colorImage.checkImage() == true)
			{
				//read in image info
				colorImage.readImageInfo();
				colorImage.numberOfPixels = (colorImage.getWidth() * colorImage.getHeight());
				nPixels = colorImage.getNumberOfPixels();

				if (checkPixelCount != 0 && nPixels > 20000)
				{
					std::cout << "Number crunching cancelled." << std::endl;
					std::cout << "This program's pixelcount is set to a maximum value of 20,000 pixels." << std::endl;
					std::cout << "The TGA-image that is specified has " << nPixels << " pixels." << std::endl;
					std::cout << "Please specify another TGA-image to work on." << std::endl;
					std::cout << std::endl;
					ppcMenu.showMainMenu();
				}

			checkPixelCount++;
			}
			//if image doesn't exist
			else
			{
				std::cout << "Attempt to open color-file failed." << std::endl;
				std::cout << std::endl;
				ppcMenu.showMainMenu();
			}
		}
		else
		{
			//if file not defined
			std::cout << "Color-file: Undefined" << std::endl;
			std::cout << std::endl;
			ppcMenu.showMainMenu();
		}
	}while(nPixels > 20000);


	//////////////////////////
	if (colorImage.getPath().size() != 0)
		{
		//read in images
		if (colorImage.checkImage() == true)
		{
			//read in image info
			colorImage.readImageInfo();
			colorImage.numberOfPixels = (colorImage.getWidth() * colorImage.getHeight());
			nPixels = colorImage.getNumberOfPixels();

			colorImage.scatterVolumeScalar.x = ppcMenu.getVolumeScalar().x;
			colorImage.scatterVolumeScalar.y = ppcMenu.getVolumeScalar().y;
			colorImage.scatterVolumeScalar.z = ppcMenu.getVolumeScalar().z;

			////////////////////////////////////////////////////
			////////////////////////////////////////////////////
			//if opacity map is defined
			if (opacityImage.getPath().size() != 0)
			{
				//if opacityImage exists
				if (opacityImage.checkImage() == true)
				{
					opacityImage.readImageInfo();
		
					//if the image is grayscale
					if (opacityImage.getChannels() == 1)
					{
						//if image is the same size as colorImage
						if (opacityImage.getWidth() == colorImage.getWidth() && opacityImage.getHeight() == colorImage.getHeight())
						{
							//write color values
							opacityImage.readAttribute(opacityExtension);
							opacityFile = true;
						}
						else
						{
							std::cout << "Opacity map: " << opacityImage.getPath() << " not the same size as TGA color image." << std::endl;
							std::cout << "Opacity map will be ignored." << std::endl;
							std::cout << "Pixel opacity defaulted to pixel opacity constant." << std::endl;
							std::cout << std::endl;

							opacityImage.setPath(colorImage.getPath());
							opacityImage.setNumberOfPixels(colorImage.numberOfPixels);
							opacityImage.readAttribute(opacityExtension, ppcMenu.getpOpacityConstant(), colorImage.numberOfPixels);
						}
					}
					else
					{
						//if image isn't a grayscale image
						std::cout << "Opacity map: " << opacityImage.getPath() << " isn't a TGA grayscale image." << std::endl;
						std::cout << "Opacity map will be ignored." << std::endl;
						std::cout << "Pixel opacity defaulted to pixel opacity constant." << std::endl;
						std::cout << std::endl;

						opacityImage.setPath(colorImage.getPath());
						opacityImage.setNumberOfPixels(colorImage.numberOfPixels);
						opacityImage.readAttribute(opacityExtension, ppcMenu.getpOpacityConstant(), colorImage.numberOfPixels);
					}
				}

				else
				{
					//if image doesn't exist
					std::cout << "Attempt to open opacity map failed." << std::endl;
					std::cout << "Pixel opacity defaulted to pixel opacity constant." << std::endl;
					std::cout << std::endl;

					opacityImage.setPath(colorImage.getPath());
					opacityImage.setNumberOfPixels(colorImage.numberOfPixels);
					opacityImage.readAttribute(opacityExtension, ppcMenu.getpOpacityConstant(), colorImage.numberOfPixels);
				}
			}
			else
			{
				//if file not defined
				opacityImage.setPath(colorImage.getPath());
				opacityImage.setNumberOfPixels(colorImage.numberOfPixels);
				opacityImage.readAttribute(opacityExtension, ppcMenu.getpOpacityConstant(), colorImage.numberOfPixels);

				std::cout << "pOpacity-file: Undefined" << std::endl;
				std::cout << "Pixel opacity defaulted to pixel opacity constant." << std::endl;
				std::cout << std::endl;
			}
	
			////////////////////////////////////////////////////
			////////////////////////////////////////////////////
			if (velocityImage.getPath().size() != 0)
			{
				//if velocityImage exists
				if (velocityImage.checkImage() == true)
				{
					velocityImage.readImageInfo();
		
					//if the image is grayscale
					if (velocityImage.getChannels() == 1)
					{
						//if image is the same size as colorImage
						if (velocityImage.getWidth() == colorImage.getWidth() && velocityImage.getHeight() == colorImage.getHeight())
						{
							//write color values
							velocityImage.readAttribute(velocityExtension);
							velocityFile = true;
						}
						else
						{
							std::cout << "Velocity scalar map: " << velocityImage.getPath() << " not the same size as TGA color image." << std::endl;
							std::cout << "Velocity scalar map will be ignored." << std::endl;
							std::cout << "Pixel velocity defaulted to pixel velocity scalar constant." << std::endl;
							std::cout << std::endl;

							velocityImage.setPath(colorImage.getPath());
							velocityImage.setNumberOfPixels(colorImage.numberOfPixels);
							velocityImage.readAttribute(velocityExtension, ppcMenu.getpVelocityConstant(), colorImage.numberOfPixels);
						}
					}
					else
					{
						//if image isn't a grayscale image
						std::cout << "Velocity scalar map: " << velocityImage.getPath() << " isn't a TGA grayscale image." << std::endl;
						std::cout << "Velocity scalar map will be ignored." << std::endl;
						std::cout << "Pixel velocity defaulted to pixel velocity scalar constant." << std::endl;
						std::cout << std::endl;

						velocityImage.setPath(colorImage.getPath());
						velocityImage.setNumberOfPixels(colorImage.numberOfPixels);
						velocityImage.readAttribute(velocityExtension, ppcMenu.getpVelocityConstant(), colorImage.numberOfPixels);
					}
				}
				else
				{
					//if image doesn't exist
					std::cout << "Attempt to open velocity scalar map failed." << std::endl;
					std::cout << "Pixel velocity defaulted to pixel velocity scalar constant." << std::endl;
					std::cout << std::endl;

					velocityImage.setPath(colorImage.getPath());
					velocityImage.setNumberOfPixels(colorImage.numberOfPixels);
					velocityImage.readAttribute(velocityExtension, ppcMenu.getpVelocityConstant(), colorImage.numberOfPixels);
				}
			}
			else
			{
				//if file not defined
				velocityImage.setPath(colorImage.getPath());
				velocityImage.setNumberOfPixels(colorImage.numberOfPixels);
				velocityImage.readAttribute(velocityExtension, ppcMenu.getpVelocityConstant(), colorImage.numberOfPixels);
				ppcMenu.setpVelocityMinConstant(ppcMenu.getpVelocityConstant());
				ppcMenu.setpVelocityMaxConstant(ppcMenu.getpVelocityConstant());

				std::cout << "pVelocity-file: Undefined" << std::endl;
				std::cout << "Pixel velocity defaulted to pixel velocity scalar constant." << std::endl;
				std::cout << std::endl;
			}	
	
			////////////////////////////////////////////////////
			////////////////////////////////////////////////////
			if (scaleImage.getPath().size() != 0)
			{
				//if scaleImage exists
				if (scaleImage.checkImage() == true)
				{
					scaleImage.readImageInfo();

					//if the image is grayscale
					if (scaleImage.getChannels() == 1)
					{
						//if image is the same size as colorImage
						if (scaleImage.getWidth() == colorImage.getWidth() && scaleImage.getHeight() == colorImage.getHeight())
						{
							//write color values
							scaleImage.readAttribute(scaleExtension);
							scaleFile = true;
						}
						else
						{
							std::cout << "Pixel scalar map: " << scaleImage.getPath() << " not the same size as TGA color image." << std::endl;
							std::cout << "Pixel scalar map will be ignored." << std::endl;
							std::cout << "Pixel scale scalar defaulted to pixel scale scalar constant." << std::endl;
							std::cout << std::endl;

							scaleImage.setPath(colorImage.getPath());
							scaleImage.setNumberOfPixels(colorImage.numberOfPixels);
							scaleImage.readAttribute(scaleExtension, ppcMenu.getpScaleConstant(), colorImage.numberOfPixels);
						}
					}
					else
					{
						//if image isn't a grayscale image
						std::cout << "Pixel scalar map: " << scaleImage.getPath() << " isn't a TGA grayscale image." << std::endl;
						std::cout << "Pixel scalar map will be ignored." << std::endl;
						std::cout << "Pixel scale scalar defaulted to pixel scale scalar constant." << std::endl;
						std::cout << std::endl;

						scaleImage.setPath(colorImage.getPath());
						scaleImage.setNumberOfPixels(colorImage.numberOfPixels);
						scaleImage.readAttribute(scaleExtension, ppcMenu.getpScaleConstant(), colorImage.numberOfPixels);
					}
				}
				else
				{
					//if image doesn't exist
					std::cout << "Attempt to open pixel scalar map failed." << std::endl;
					std::cout << "Pixel scale scalar defaulted to pixel scale scalar constant." << std::endl;
					std::cout << std::endl;

					scaleImage.setPath(colorImage.getPath());
					scaleImage.setNumberOfPixels(colorImage.numberOfPixels);
					scaleImage.readAttribute(scaleExtension, ppcMenu.getpScaleConstant(), colorImage.numberOfPixels);
				}
			}
			else
			{
				//if file not defined
				scaleImage.setPath(colorImage.getPath());
				scaleImage.setNumberOfPixels(colorImage.numberOfPixels);
				scaleImage.readAttribute(scaleExtension, ppcMenu.getpScaleConstant(), colorImage.numberOfPixels);

				std::cout << "pScalar-file: Undefined" << std::endl;
				std::cout << "Pixel scale scalar defaulted to pixel scale scalar constant." << std::endl;
				std::cout << std::endl;
			}

			////////////////////////////////////////////////////
			////////////////////////////////////////////////////
			if (heightImage.getPath().size() != 0)
			{
				//if heightImage exists
				if (heightImage.checkImage() == true)
				{
					heightImage.readImageInfo();	

					//if the image is grayscale
					if (heightImage.getChannels() == 1)
					{
						//if image is the same size as colorImage
						if (heightImage.getWidth() == colorImage.getWidth() && heightImage.getHeight() == colorImage.getHeight())
						{
							//write color values
							heightImage.readAttribute(heightExtension);
							heightFile = true;
						}
						else
						{
							std::cout << "Height map: " << heightImage.getPath() << " not the same size as TGA color image." << std::endl;
							std::cout << "Height map will be ignored." << std::endl;
							std::cout << "Pixel height offset defaulted to pixel height offset constant." << std::endl;
							std::cout << std::endl;

							heightImage.setPath(colorImage.getPath());
							heightImage.setNumberOfPixels(colorImage.numberOfPixels);
							heightImage.readAttribute(heightExtension, ppcMenu.getpHeightMaxConstant(), colorImage.numberOfPixels);
						}
					}
					else
					{
						//if image isn't a grayscale image
						std::cout << "Height map: " << heightImage.getPath() << " isn't a TGA grayscale image." << std::endl;
						std::cout << "Height map will be ignored." << std::endl;
						std::cout << "Pixel height offset defaulted to pixel height offset constant." << std::endl;
						std::cout << std::endl;

						heightImage.setPath(colorImage.getPath());
						heightImage.setNumberOfPixels(colorImage.numberOfPixels);
						heightImage.readAttribute(heightExtension, ppcMenu.getpHeightMaxConstant(), colorImage.numberOfPixels);
					}
				}
				else
				{
					//if image doesn't exist
					std::cout << "Attempt to open height map failed." << std::endl;
					std::cout << "Pixel height offset defaulted to pixel height offset constant." << std::endl;
					std::cout << std::endl;

					heightImage.setPath(colorImage.getPath());
					heightImage.setNumberOfPixels(colorImage.numberOfPixels);
					heightImage.readAttribute(heightExtension, ppcMenu.getpHeightMaxConstant(), colorImage.numberOfPixels);
				}
			}
			else
			{
				//if file not defined
				heightImage.setPath(colorImage.getPath());
				heightImage.setNumberOfPixels(colorImage.numberOfPixels);
				heightImage.readAttribute(heightExtension, ppcMenu.getpHeightMaxConstant(), colorImage.numberOfPixels);

				std::cout << "pHeight-file: Undefined" << std::endl;
				std::cout << "Pixel height offset defaulted to pixel height offset constant." << std::endl;
				std::cout << std::endl;
			}

			////////////////////////////////////////////////////
			//calculate velocity on a per pixel basis by reading in the pVelocity-file
			std::string pVelocityTemp = (velocityImage.getPath() + velocityExtension);
			char* ptrVelocityPath = &pVelocityTemp[0];
			ppcVelocity.open (ptrVelocityPath);
	
			if (ppcVelocity.good() == true)
			{
				std::cout << "Calculating per pixel velocity...";
				ppcVelocity >> nPixels;
				float * velocityArray;
				velocityArray = new float[nPixels];
			
				for (unsigned int i = 0; i < nPixels; i++)
				{
					ppcVelocity >> velocityArray[i];
					velocityArray[i] = (((velocityArray[i] / 255) * (ppcMenu.getpVelocityMaxConstant() - ppcMenu.getpVelocityMinConstant())) + ppcMenu.getpVelocityMinConstant());
	
					//prevent that velocity = 0.0 -> no movement -> infinite loop
					if (velocityArray[i] < 0.01)
					{
						velocityArray[i] = 0.01;
					}
					//to fix the velocity to 2 decimal places
					velocityArray[i] = static_cast <int> (velocityArray[i] * 1000);
					velocityArray[i] = static_cast <float> (velocityArray[i] / 1000);

					//assign per pixel velocity to pixels
					colorImage.pixels[i].setVelocityScalar(velocityArray[i]);
				}
				ppcVelocity.close();
				std::cout << "done" << std::endl;
			}

			////////////////////////////////////////////////////
			colorImage.readAttribute(colorExtension);
			colorImage.createPixels();

			////////////////////////////////////////////////////
			//calculate heightmap
			std::string pHeightTemp = (heightImage.getPath() + heightExtension);
			char* ptrHeightPath = &pHeightTemp[0];
			ppcHeight.open (ptrHeightPath);
	
			if (ppcHeight.good() == true)
			{
				std::cout << "Calculating per pixel height offset...";
				ppcHeight >> nPixels;
				heightArray = new float[nPixels];
				vector vTemp(0, 0, 0);

				colorImage.setDepth(ppcMenu.getpHeightMaxConstant());
			
				for (unsigned int i = 0; i < nPixels; i++)
				{
					ppcHeight >> heightArray[i];
					heightArray[i] = static_cast <int>((heightArray[i] / 255) * ppcMenu.getpHeightMaxConstant());
					vTemp = colorImage.pixels[i].getCurrentPosition();
					vTemp.z = heightArray[i];
					colorImage.pixels[i].setCurrentPosition(vTemp);
					colorImage.pixels[i].setFinalPosition(vTemp);
				}
				ppcHeight.close();
				std::cout << "done" << std::endl;
			}
			else
			{
				heightArray = new float[nPixels];
				for (unsigned int i = 0; i < nPixels; i++)
				{
					heightArray[i] = 0.0;
				}
			}

			std::cout << "Scattering pixels...";
			colorImage.scatterPixels();
			std::cout << "done" << std::endl;
			colorImage.movePixels2();
		}
	
		//if image doesn't exist
		else
		{
			std::cout << "Attempt to open color-file failed." << std::endl;
			std::cout << std::endl;
			ppcMenu.showMainMenu();
		}
	}
	else
	{
		//if file not defined
		std::cout << "Color-file: Undefined" << std::endl;
		std::cout << std::endl;
		ppcMenu.showMainMenu();
	}

	std::cout << "Processing time: " << ((static_cast <float> (GetTickCount()) - static_cast <float> (tickCount)) / 1000) << " seconds" << std::endl;
	std::cout << std::endl;
	std::cout << "Thank you for using PPC-cruncher. Please come again." << std::endl;
	std::cout << "Press <ENTER> to exit." << std::endl;
	std::cin.get();
}