#define DO_OUTPUT 0
#pragma once
#include <iostream>
#include <fstream>
#include <time.h>
#include "wrapper.h"

////////////////////////////////////////////////////////////////////////////////////
//	default values for static members
//	ONLY CHANGE THESE LINES, DO NOT DELETE THEM!!
//	Static members must be initialized.
//	Since a header file can have more then 1 class definition in it,
//	we must provide the correct classname specification
vector wrapper::imageOffset (0, 0, 0);
vector wrapper::pixelSpacing (1, 1, 1);
vector wrapper::scatterVolumeScalar (2, 2, 2);
vector wrapper::pixelScalar (1, 1, 1);
vector wrapper::scatterVolumeDimension (1, 1, 1);

////////////////////////////////////////////////////////////////////////////////////
wrapper::wrapper()
{
	// Default constructor, 
	width = 0;
	height = 0;
	depth = 1;
	pixelDepth = 0;
	// ALWAYS INITIALIZE POINTERS TO NULL!!!
	pixels = NULL;
	colorMap = "";
}

////////////////////////////////////////////////////////////////////////////////////
//Constructor that takes a tga-object
wrapper::wrapper (LTGA tga)
{
	pixels = NULL;
	// Load the actual image
	loadImage(tga);
}

////////////////////////////////////////////////////////////////////////////////////
// We're using pointers so we seriously need a destructor to clean up
wrapper::~wrapper ()	
{
	// BEWARE: Before deleting an object (the object being pixels) we 
	//   need to be absolutly sure that the object does exist.
	if (pixels != NULL)
	{
		// Release the memory allocated to pixels
		delete [] pixels;
	}
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::setNumberOfPixels(unsigned int nPixels)
{
	numberOfPixels = nPixels;
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::setDepth(float iDepth)
{
	depth = iDepth;
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::loadImage (LTGA &tga)
{
	byte *iPixel;
	unsigned int pixelCount = 0;

	// getImageWidth() returns an unsigned int; our width is a float
	//   so a conversion is at the order.
	width = float(tga.getImageWidth());
	height = float(tga.getImageHeight());
	depth = 1;
	pixelDepth = tga.getPixelDepth();
	//+2 to give the pixels extra space, must the scatterVolumeScalar be set to (1, 1, 1).
	vector scatterVolumeDimension ((width * scatterVolumeScalar.x * pixelSpacing.x + 2), (height * scatterVolumeScalar.y * pixelSpacing.y + 2), (depth * scatterVolumeScalar.z * pixelSpacing.z + 2));
	//create pointer-array for pixels
	if (pixels != NULL)
	{
		delete [] pixels;
	}

	// Create the objects
	pixels = new pixel[getNumberOfPixels()];
	iPixel = tga.getPixels();

	if (getChannels() == 1)
	{
		// Instead of always defining 'i++' as the incremental part
		//   of the for-declaration, it's also a perfectly possible to 
		//   define the incremental part as a function or a complete calculation
		for (unsigned int i = 0; i < (getNumberOfPixels() * getChannels()); i = i + getChannels())
		{
			pixels[pixelCount].rgba.red = iPixel[i];
			pixels[pixelCount].rgba.green = iPixel[i];
			pixels[pixelCount].rgba.blue = iPixel[i];
			pixels[pixelCount].rgba.alpha = 255;
			pixelCount++;
		}
	}
	//RGB-RGBA image
	else if (getChannels() == 3 || getChannels() == 4)
	{
		for (unsigned int i = 0; i < (getNumberOfPixels() * getChannels()); i = i + getChannels())
		{
			pixels[pixelCount].rgba.red = iPixel[i];
			pixels[pixelCount].rgba.green = iPixel[i + 1];
			pixels[pixelCount].rgba.blue = iPixel[i + 2];
			pixels[pixelCount].rgba.alpha = 255;
			pixelCount++;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////
bool wrapper::checkImage()
{
#if DO_OUTPUT == 1
	std::cout << "Entering checkImage procedure..." << std::endl;
#endif

	LTGA image;
	image.loadFromFile(getPath());

	if (image.isLoaded() == true)
	{
#if DO_OUTPUT == 1
		std::cout << "Specified image loaded successfully." << std::endl;
		std::cout << "Exiting checkImage procedure..." << std::endl;
		std::cout <<"----------------------------------------" << std::endl;
#endif
		return true;
	}
	else
	{
#if DO_OUTPUT == 1
		std::cout << "Specified image failed to load." << std::endl;
		std::cout << "Exiting checkImage procedure..." << std::endl;
		std::cout <<"----------------------------------------" << std::endl;
#endif
		return false;
	}

#if DO_OUTPUT == 1
	std::cout << "Exiting checkImage procedure..." << std::endl;
#endif
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::readImageInfo()
{
#if DO_OUTPUT == 1
	std::cout << "Entering readImageInfo procedure..." << std::endl;
#endif
	LTGA image;
	
	image.loadFromFile(getPath());

	// Load the image into the wrapper class
	loadImage(image);

	//print image info
	std::cout <<"----------------------------------------" << std::endl;
	std::cout << "Image:			" << getPath() << std::endl;
	std::cout << "Image size:		" << getWidth() << " x " << getHeight() << std::endl;
	std::cout << "# of pixels:		" << getNumberOfPixels() << std::endl;
	std::cout << "Pixel depth:		" << getPixelDepth() << std::endl;
	std::cout << "# of channels:\t	" << getChannels() << std::endl;
	std::cout <<"----------------------------------------" << std::endl;

#if DO_OUTPUT == 1
	std::cout << "Exiting readImageInfo procedure..." << std::endl;
	std::cout <<"----------------------------------------" << std::endl;
#endif
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::createPixels()
{
#if DO_OUTPUT == 1
	std::cout << "Entering createPixels procedure..." << std::endl;
#endif
	//create pixels
	unsigned int pixelCount = 0;

#if DO_OUTPUT == 1
	std::cout << "Initial pixel positions:" << std::endl;
#endif

	for (int j = 0; j < getHeight(); j++)
	{
		for (int i = 0; i < getWidth(); i++)
		{
			vector vTemp;
			vTemp.x = ((((i - (getWidth() / 2.0f)) * pixelSpacing.x) + (pixelSpacing.x / 2.0f)) + imageOffset.x);
			vTemp.y = ((((j - (getHeight() / 2.0f)) * pixelSpacing.y) + (pixelSpacing.y / 2.0f)) + imageOffset.y);
			vTemp.z = imageOffset.z;
			pixels[pixelCount].setCurrentPosition (vTemp);
			pixels[pixelCount].setNextPosition (vTemp);
			pixels[pixelCount].setFinalPosition (vTemp);
#if DO_OUTPUT == 1
			std::cout << vTemp << std::endl;
#endif
			pixelCount++;
		}
	}

#if DO_OUTPUT == 1
	std::cout << "Exiting createPixels procedure..." << std::endl;
	std::cout <<"----------------------------------------" << std::endl;
#endif
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::readAttribute(std::string &fileExtension)
{
#if DO_OUTPUT == 1
	std::cout << "Entering readAttribute procedure..." << std::endl;
#endif
	//create pixels
	unsigned int pixelCount = 0;
	std::string outputAttributeDataPath = (getPath() + fileExtension);
	char* ptrAttributePath = &outputAttributeDataPath[0];

	//create and open .color-file
	std::ofstream outputAttributeData;
	outputAttributeData.open (ptrAttributePath);

	outputAttributeData << getNumberOfPixels() << std::endl;

	for (int k = 0; k < getDepth(); k++)
	{
		for (int j = 0; j < getHeight(); j++)
		{
			for (int i = 0; i < getWidth(); i++)
			{
				if (getChannels() == 1)
				{
					outputAttributeData << static_cast <int> (pixels[pixelCount].rgba.red) << std::endl;
					pixelCount++;
				}
				else
				{
					//write pixel color to attribute-file
					outputAttributeData << static_cast <int> (pixels[pixelCount].rgba.red) << " ";
					outputAttributeData << static_cast <int> (pixels[pixelCount].rgba.green) << " ";
					outputAttributeData << static_cast <int> (pixels[pixelCount].rgba.blue) << " ";
					outputAttributeData << static_cast <int> (pixels[pixelCount].rgba.alpha) << " ";
					outputAttributeData << std::endl;
					pixelCount++;
				}
			}
		}
	}

	//closing .attribute-file
	outputAttributeData.close();

#if DO_OUTPUT == 1
	std::cout << "Exiting readAttribute procedure..." << std::endl;
	std::cout <<"----------------------------------------" << std::endl;
#endif
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::readAttribute(std::string &fileExtension, float constant, unsigned int &nPixels)
{
#if DO_OUTPUT == 1
	std::cout << "Entering readAttribute procedure..." << std::endl;
#endif
	//create pixels
	unsigned int pixelCount = 0;
	std::string outputAttributeDataPath = (getPath() + fileExtension);
	char* ptrAttributePath = &outputAttributeDataPath[0];

	//create and open .color-file
	std::ofstream outputAttributeData;
	outputAttributeData.open (ptrAttributePath);

	outputAttributeData << nPixels << std::endl;

	for (unsigned int i = 0; i < nPixels; i++)
	{
		outputAttributeData << constant << std::endl;
		pixelCount++;
	}

	//closing .attribute-file
	outputAttributeData.close();

#if DO_OUTPUT == 1
	std::cout << "Exiting readAttribute procedure..." << std::endl;
	std::cout <<"----------------------------------------" << std::endl;
#endif
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::scatterPixels()
{
#if DO_OUTPUT == 1
	std::cout << "Entering scatterPixels procedure..." << std::endl;
#endif

	vector pixelScatterPositionTemp;
	vector pixelScatterPosition;
	int equalNumbers = 1;

	if (scatterVolumeScalar.x < 1.0)
	{
		scatterVolumeScalar.x = 1.0;
		std::cout << "scatterVolumeScalar.x was less than 1.0. Defaulted to 1.0." << std::endl;
	}

	if (scatterVolumeScalar.y < 1.0)
	{
		scatterVolumeScalar.y = 1.0;
		std::cout << "scatterVolumeScalar.y was less than 1.0. Defaulted to 1.0." << std::endl;
	}

	if (scatterVolumeScalar.z < 1.0)
	{
		scatterVolumeScalar.z = 1.0;
		std::cout << "scatterVolumeScalar.z was less than 1.0. Defaulted to 1.0." << std::endl;
	}

	scatterVolumeDimension.x = getWidth() * scatterVolumeScalar.x * pixelSpacing.x + 2;
	scatterVolumeDimension.y = getHeight() * scatterVolumeScalar.y * pixelSpacing.y + 2;
	scatterVolumeDimension.z = getDepth() * scatterVolumeScalar.z * pixelSpacing.z + 2;

#if DO_OUTPUT == 1
	std::cout << "Initial pixel scatter positions:" << std::endl;
#endif

	//Seed the random-number generator with current time so that the numbers will be different every time we run.
	srand ((unsigned) time(NULL));
	const float fRandMax = RAND_MAX;

	for (uint i = 0; i < getNumberOfPixels(); i++)
	{
		//RAND_MAX = 32767 -> range for rand()
		pixelScatterPositionTemp.x = static_cast <float> (static_cast <int> (((rand()) * getWidth() * scatterVolumeScalar.y)/fRandMax));
		pixelScatterPositionTemp.y = static_cast <float> (static_cast <int> (((rand()) * getHeight() * scatterVolumeScalar.y)/fRandMax));
		pixelScatterPositionTemp.z = static_cast <float> (static_cast <int> (((rand()) * getDepth() * scatterVolumeScalar.z)/fRandMax));

		pixelScatterPositionTemp.x = ((pixelScatterPositionTemp.x * pixelSpacing.x) + imageOffset.x);
		pixelScatterPositionTemp.y = ((pixelScatterPositionTemp.y * pixelSpacing.y) + imageOffset.y);
		pixelScatterPositionTemp.z = ((pixelScatterPositionTemp.z * pixelSpacing.z) + imageOffset.z);

		pixelScatterPosition.x = (pixelScatterPositionTemp.x - (scatterVolumeDimension.x / 2) + (pixelSpacing.x / 2));
		pixelScatterPosition.y = (pixelScatterPositionTemp.y - (scatterVolumeDimension.y / 2) + (pixelSpacing.y / 2));
		pixelScatterPosition.z = (pixelScatterPositionTemp.z - (scatterVolumeDimension.z / 2) + (pixelSpacing.z / 2));

		if (static_cast <int> (getWidth()) % 2 == 1)
		{
		pixelScatterPosition.x = (pixelScatterPositionTemp.x - (scatterVolumeDimension.x / 2));
		}

		if (static_cast <int> (getHeight()) % 2 == 1)
		{
		pixelScatterPosition.y = (pixelScatterPositionTemp.y - (scatterVolumeDimension.y / 2));
		}

		if (static_cast <int> (getDepth()) % 2 == 1)
		{
		pixelScatterPosition.z = (pixelScatterPositionTemp.z - (scatterVolumeDimension.z / 2));
		}

		pixels[i].setCurrentPosition(pixelScatterPosition);
		pixels[i].setNextPosition(pixelScatterPosition);

#if DO_OUTPUT == 1
		std::cout << "pixel-ID: " << i << "\t" << pixels[i].getCurrentPosition() << std::endl;
#endif
	}

#if DO_OUTPUT == 1
	std::cout <<"----------------------------------------" << std::endl;
#endif

	while (equalNumbers > 0)
	{
		equalNumbers = 0;

		for (unsigned int i = 0; i < (getNumberOfPixels() - 1); i++)
		{
			for (unsigned int j = (i + 1); j < getNumberOfPixels();)
			{
				if (pixels[i].getCurrentPosition() == pixels[j].getCurrentPosition())
				{
#if DO_OUTPUT == 1
					std::cout << "Equal pixel positions: " << i << "-" << j << std::endl;
#endif

					const float fRandMax = RAND_MAX;

					pixelScatterPositionTemp.x = static_cast <float> (static_cast <int> (((rand()) * getWidth() * scatterVolumeScalar.y)/fRandMax));
					pixelScatterPositionTemp.y = static_cast <float> (static_cast <int> (((rand()) * getHeight() * scatterVolumeScalar.y)/fRandMax));
					pixelScatterPositionTemp.z = static_cast <float> (static_cast <int> (((rand()) * getDepth() * scatterVolumeScalar.z)/fRandMax));

					pixelScatterPositionTemp.x = ((pixelScatterPositionTemp.x * pixelSpacing.x) + imageOffset.x);
					pixelScatterPositionTemp.y = ((pixelScatterPositionTemp.y * pixelSpacing.y) + imageOffset.y);
					pixelScatterPositionTemp.z = ((pixelScatterPositionTemp.z * pixelSpacing.z) + imageOffset.z);

					pixelScatterPosition.x = (pixelScatterPositionTemp.x - (scatterVolumeDimension.x / 2) + (pixelSpacing.x / 2));
					pixelScatterPosition.y = (pixelScatterPositionTemp.y - (scatterVolumeDimension.y / 2) + (pixelSpacing.y / 2));
					pixelScatterPosition.z = (pixelScatterPositionTemp.z - (scatterVolumeDimension.z / 2) + (pixelSpacing.z / 2));

					if (static_cast <int> (getWidth()) % 2 == 1)
					{
					pixelScatterPosition.x = (pixelScatterPositionTemp.x - (scatterVolumeDimension.x / 2));
					}

					if (static_cast <int> (getHeight()) % 2 == 1)
					{
					pixelScatterPosition.y = (pixelScatterPositionTemp.y - (scatterVolumeDimension.y / 2));
					}

					if (static_cast <int> (getDepth()) % 2 == 1)
					{
					pixelScatterPosition.z = (pixelScatterPositionTemp.z - (scatterVolumeDimension.z / 2));
					}

					pixels[i].setCurrentPosition(pixelScatterPosition);
					pixels[i].setNextPosition(pixelScatterPosition);

					equalNumbers++;
				}
	
				if (pixels[i].getCurrentPosition() != pixels[j].getCurrentPosition())
				{
					j++;
				}
			}
		}

		if (equalNumbers > 0)
		{
			equalNumbers = 1;
		}
		else
		{
			equalNumbers = 0;
		}
	}

#if DO_OUTPUT == 1
	std::cout <<"----------------------------------------" << std::endl;
	std::cout << "Final pixel scatter positions:" << std::endl;

	for (unsigned int i = 0; i < getNumberOfPixels(); i++)
	{
		std::cout << "pixel-ID: " << i << "\t" << pixels[i].getCurrentPosition() << std::endl;

		//check if scatter position is the same as the final position. if so, set "checkPosition" to true
		if (pixels[i].checkPosition() == true)
		{
			std::cout << "Scatter position = final position." << std::endl;
		}
	}

	std::cout << "Exiting scatterPixels procedure..." << std::endl;
	std::cout <<"----------------------------------------" << std::endl;
#endif
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::movePixels ()
{
#if DO_OUTPUT == 1
	std::cout << "Entering movePixels procedure..." << std::endl;
#endif

	int frameNumber = 0;
	int pixelsInPosition = 1;
	std::string outputPositionDataPath = (colorMap + ".pPath");
	char* ptrPosPath = &outputPositionDataPath[0];

	//create and open .path-file
	std::ofstream outputPositionData;
	outputPositionData.open (ptrPosPath);

	outputPositionData << getNumberOfPixels() << " " << frameNumber << "\n" << std::endl;

	while (pixelsInPosition > 0)
	{
		pixelsInPosition = 0;

		//set the pixels in their new position
		for (unsigned int i = 0; i < getNumberOfPixels(); i++)
		{
			pixels[i].setCurrentPosition(pixels[i].getNextPosition());
			pixels[i].setVelocity (0.0, 0.0, 0.0);

			if (pixels[i].getCurrentPosition() != pixels[i].getFinalPosition())
			{
				pixelsInPosition++;
			}
		}

#if DO_OUTPUT == 1
		std::cout << "------------ frame " << frameNumber << " ------------" << std::endl;
#endif

		//print frame number in file
		std::cout << "Calculating frame " << frameNumber << "...";
		outputPositionData << frameNumber << std::endl;

		//calculate every pixels' new position and velocity
		for (unsigned int i = 0; i < getNumberOfPixels(); i++)
		{
#if DO_OUTPUT == 1
			std::cout << "Difference vector: " << pixels[i].getPositionDifference() << std::endl;
#endif
			float high = -100000.0;
			float velocityComponent = 0.0;
			vector vNull (0.0, 0.0, 0.0);

			//check if the resulting velocity vector isn't gonna be 0 -> pixels would never move -> infinite loop
			if (pixels[i].getVelocityScalar() == 0)
			{
				float scalar = static_cast <float> (0.01);
				std::cout << "Pixel velocity scalar was set to 0.0. Defaulted to 1.0." << std::endl;
				pixels[i].setVelocityScalar(scalar);
			}

			///////////////////////////X-component///////////////////////////
			if (pixels[i].getPositionDifference().x > high)
			{
				//check for the highest component
				high = pixels[i].getPositionDifference().x;

				//check if the pixel is to the left or the right of its final position and modify the sign of
				//velocity accordingly
				if ((pixels[i].getFinalPosition().x - pixels[i].getCurrentPosition().x) > 0)
				{
					velocityComponent = (1 * pixels[i].getVelocityScalar());
				}
				else
				{
					velocityComponent = (-1 * pixels[i].getVelocityScalar());
				}

				if (pixels[i].getPositionDifference() == vNull)
				{
					velocityComponent = 0.0;
				}

				//if the difference in X is higher than the velocity in that direction (so the pixel would go
				//past its X-destination, it modifies the velocity-component to match the current X-difference
				if (velocityComponent > pixels[i].getPositionDifference().x)
				{
					velocityComponent = (pixels[i].getFinalPosition().x - pixels[i].getCurrentPosition().x);
				}

				//set the velocity vector
				pixels[i].setVelocity(velocityComponent, 0, 0);
				//set the next-position vector
				pixels[i].setNextPosition((pixels[i].getCurrentPosition() + pixels[i].getVelocity()));
			}

			///////////////////////////Y-component///////////////////////////
			if (pixels[i].getPositionDifference().y > high)
			{
				//check for the highest component
				high = pixels[i].getPositionDifference().y;

				//check if the pixel is to the left or the right of its final position and modify the sign of
				//velocity accordingly
				if ((pixels[i].getFinalPosition().y - pixels[i].getCurrentPosition().y) > 0)
				{
					velocityComponent = (1 * pixels[i].getVelocityScalar());
				}
				else
				{
					velocityComponent = (-1 * pixels[i].getVelocityScalar());
				}

				if (pixels[i].getPositionDifference() == vNull)
				{
					velocityComponent = 0.0;
				}

				//if the difference in Y is higher than the velocity in that direction (so the pixel would go
				//past its Y-destination, it modifies the velocity-component to match the current Y-difference
				if (velocityComponent > pixels[i].getPositionDifference().y)
				{
					//velocityComponent = (velocityComponent / (abs (velocityComponent))) * high;
					velocityComponent = (pixels[i].getFinalPosition().y - pixels[i].getCurrentPosition().y);
				}

				//set the velocity vector
				pixels[i].setVelocity(0, velocityComponent, 0);
				//set the next-position vector
				pixels[i].setNextPosition((pixels[i].getCurrentPosition() + pixels[i].getVelocity()));
			}

			///////////////////////////Z-component///////////////////////////
			if (pixels[i].getPositionDifference().z > high)
			{
				//check for the highest component
				high = pixels[i].getPositionDifference().z;

				//check if the pixel is to the left or the right of its final position and modify the sign of
				//velocity accordingly
				if ((pixels[i].getFinalPosition().z - pixels[i].getCurrentPosition().z) > 0)
				{
					velocityComponent = (1 * pixels[i].getVelocityScalar());
				}
				else
				{
					velocityComponent = (-1 * pixels[i].getVelocityScalar());
				}

				if (pixels[i].getPositionDifference() == vNull)
				{
					velocityComponent = 0.0;
				}

				//if the difference in Z is higher than the velocity in that direction (so the pixel would go
				//past its Z-destination, it modifies the velocity-component to match the current Z-difference
				if (velocityComponent > pixels[i].getPositionDifference().z)
				{
					//velocityComponent = (velocityComponent / (abs (velocityComponent))) * high;
					velocityComponent = (pixels[i].getFinalPosition().z - pixels[i].getCurrentPosition().z);
				}

				//set the velocity vector
				pixels[i].setVelocity(0, 0, velocityComponent);
				//set the next-position vector
				pixels[i].setNextPosition((pixels[i].getCurrentPosition() + pixels[i].getVelocity()));
			}

#if DO_OUTPUT == 1
			///////////////////////////print info///////////////////////////
			std::cout << "Velocity: " << pixels[i].getVelocity() << std::endl;
			std::cout << "Current position: " << pixels[i].getCurrentPosition() << std::endl;
			std::cout << "Next position: " << pixels[i].getNextPosition() << std::endl;
			std::cout << std::endl;
#endif

			//write out to .path-file
			outputPositionData.precision(4);
			outputPositionData << pixels[i].getCurrentPosition().x << " ";
			outputPositionData << pixels[i].getCurrentPosition().y << " ";
			outputPositionData << pixels[i].getCurrentPosition().z << std::endl;

		}

		std::cout << "done" << std::endl;
	frameNumber++;
	}

	//rewind file to write numberOfPixels and numberOfFrames
	outputPositionData.seekp(0);
	outputPositionData << getNumberOfPixels() << " " << frameNumber;// << std::endl;
	//closing .path-file
	outputPositionData.close();

#if DO_OUTPUT == 1
	std::cout << "Exiting movePixels procedure..." << std::endl;
	std::cout <<"----------------------------------------" << std::endl;
#endif
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::movePixels2()
{
#if DO_OUTPUT == 1
	std::cout << "Entering movePixels procedure..." << std::endl;
#endif

	int frameNumber = 0;
	int pixelsInPosition = 1;
	std::string outputPositionDataPath = (colorMap + ".pPath");
	char* ptrPosPath = &outputPositionDataPath[0];

	//create and open .path-file
	std::ofstream outputPositionData;
	outputPositionData.open (ptrPosPath);

	outputPositionData << getNumberOfPixels() << " " << frameNumber << "\n" << std::endl;

	while (pixelsInPosition > 0)
	{
		pixelsInPosition = 0;

		//set the pixels in their new position
		for (unsigned int i = 0; i < getNumberOfPixels(); i++)
		{
			pixels[i].setCurrentPosition(pixels[i].getNextPosition());
			pixels[i].setVelocity (0.0, 0.0, 0.0);

			if (pixels[i].getCurrentPosition() != pixels[i].getFinalPosition())
			{
				pixelsInPosition++;
			}
		}

#if DO_OUTPUT == 1
		std::cout << "------------ frame " << frameNumber << " ------------" << std::endl;
#endif

		//print frame number in file
		std::cout << "Calculating frame " << frameNumber << "...";
		outputPositionData << frameNumber << std::endl;

		//calculate every pixels' new position and velocity
		for (unsigned int i = 0; i < getNumberOfPixels(); i++)
		{
#if DO_OUTPUT == 1
			std::cout << "Difference vector: " << pixels[i].getPositionDifference() << std::endl;
#endif
			float high = -100000.0;
			float velocityComponent = 0.0;
			vector vNull (0.0, 0.0, 0.0);

			//check if the resulting velocity vector isn't gonna be 0 -> pixels would never move -> infinite loop
			if (pixels[i].getVelocityScalar() == 0)
			{
				float scalar = static_cast <float> (0.01);
				std::cout << "Pixel velocity scalar was set to 0.0. Defaulted to 1.0." << std::endl;
				pixels[i].setVelocityScalar(scalar);
			}

			///////////////////////////X-component///////////////////////////
			if (pixels[i].getPositionDifference().x > high)
			{
				//check for the highest component
				high = pixels[i].getPositionDifference().x;

				//check if the pixel is to the left or the right of its final position and modify the sign of
				//velocity accordingly
				if ((pixels[i].getFinalPosition().x - pixels[i].getCurrentPosition().x) > 0)
				{
					velocityComponent = (1 * pixels[i].getVelocityScalar());
				}
				else
				{
					velocityComponent = (-1 * pixels[i].getVelocityScalar());
				}

				if (pixels[i].getPositionDifference() == vNull)
				{
					velocityComponent = 0.0;
				}

				//if the difference in X is higher than the velocity in that direction (so the pixel would go
				//past its X-destination, it modifies the velocity-component to match the current X-difference
				if (velocityComponent > pixels[i].getPositionDifference().x)
				{
					velocityComponent = (pixels[i].getFinalPosition().x - pixels[i].getCurrentPosition().x);
				}

				//set the velocity vector
				pixels[i].setVelocity(velocityComponent, 0, 0);
				//set the next-position vector
				pixels[i].setNextPosition((pixels[i].getCurrentPosition() + pixels[i].getVelocity()));
			}

			///////////////////////////Y-component///////////////////////////
			if (pixels[i].getPositionDifference().y > high)
			{
				//check for the highest component
				high = pixels[i].getPositionDifference().y;

				//check if the pixel is to the left or the right of its final position and modify the sign of
				//velocity accordingly
				if ((pixels[i].getFinalPosition().y - pixels[i].getCurrentPosition().y) > 0)
				{
					velocityComponent = (1 * pixels[i].getVelocityScalar());
				}
				else
				{
					velocityComponent = (-1 * pixels[i].getVelocityScalar());
				}

				if (pixels[i].getPositionDifference() == vNull)
				{
					velocityComponent = 0.0;
				}

				//if the difference in Y is higher than the velocity in that direction (so the pixel would go
				//past its Y-destination, it modifies the velocity-component to match the current Y-difference
				if (velocityComponent > pixels[i].getPositionDifference().y)
				{
					//velocityComponent = (velocityComponent / (abs (velocityComponent))) * high;
					velocityComponent = (pixels[i].getFinalPosition().y - pixels[i].getCurrentPosition().y);
				}

				//set the velocity vector
				pixels[i].setVelocity(0, velocityComponent, 0);
				//set the next-position vector
				pixels[i].setNextPosition((pixels[i].getCurrentPosition() + pixels[i].getVelocity()));
			}

			///////////////////////////Z-component///////////////////////////
			if (pixels[i].getPositionDifference().z > high)
			{
				//check for the highest component
				high = pixels[i].getPositionDifference().z;

				//check if the pixel is to the left or the right of its final position and modify the sign of
				//velocity accordingly
				if ((pixels[i].getFinalPosition().z - pixels[i].getCurrentPosition().z) > 0)
				{
					velocityComponent = (1 * pixels[i].getVelocityScalar());
				}
				else
				{
					velocityComponent = (-1 * pixels[i].getVelocityScalar());
				}

				if (pixels[i].getPositionDifference() == vNull)
				{
					velocityComponent = 0.0;
				}

				//if the difference in Z is higher than the velocity in that direction (so the pixel would go
				//past its Z-destination, it modifies the velocity-component to match the current Z-difference
				if (velocityComponent > pixels[i].getPositionDifference().z)
				{
					//velocityComponent = (velocityComponent / (abs (velocityComponent))) * high;
					velocityComponent = (pixels[i].getFinalPosition().z - pixels[i].getCurrentPosition().z);
				}

				//set the velocity vector
				pixels[i].setVelocity(0, 0, velocityComponent);
				//set the next-position vector
				pixels[i].setNextPosition((pixels[i].getCurrentPosition() + pixels[i].getVelocity()));
			}

			//check if next position is blocked by another pixel
			for (unsigned int j = (i + 1); j < (getNumberOfPixels() - 1); j++)
			{
				vector posTemp(0, 0, 0);
				posTemp = pixels[j].getNextPosition();
#if DO_OUTPUT == 1
				if (pixels[i].getNextPosition() == posTemp)
				{
					std::cout << "Possible collision detected." << std::endl;
					std::cout << "current pixel: " << i << std::endl;
					std::cout << "testing pixel: " << j << std::endl;
				}
#endif
			}

#if DO_OUTPUT == 1
			///////////////////////////print info///////////////////////////
			std::cout << "Velocity: " << pixels[i].getVelocity() << std::endl;
			std::cout << "Current position: " << pixels[i].getCurrentPosition() << std::endl;
			std::cout << "Next position: " << pixels[i].getNextPosition() << std::endl;
			std::cout << std::endl;
#endif

			//write out to .path-file
			outputPositionData.precision(4);
			outputPositionData << pixels[i].getCurrentPosition().x << " ";
			outputPositionData << pixels[i].getCurrentPosition().y << " ";
			outputPositionData << pixels[i].getCurrentPosition().z << std::endl;

		}

		std::cout << "done" << std::endl;
	frameNumber++;
	}

	//rewind file to write numberOfPixels and numberOfFrames
	outputPositionData.seekp(0);
	outputPositionData << getNumberOfPixels() << " " << frameNumber;// << std::endl;
	//closing .path-file
	outputPositionData.close();

#if DO_OUTPUT == 1
	std::cout << "Exiting movePixels procedure..." << std::endl;
	std::cout <<"----------------------------------------" << std::endl;
#endif
}