#include <windows.h>
#include <GL/glut.h>
#include <iostream>
#include <string>

////////////////////////////////////////////////////////////////////////////////////

bool LMB;
bool MMB;
bool RMB;

int	lastPosX;
int lastPosY;
int stepX;
int stepY;

float cameraZoom;
float cameraTransX;
float cameraTransY;
float cameraSpinX;
float cameraSpinY;

////////////////////////////////////////////////////////////////////////////////////
void init()
{
	//background color
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	//	Enable depth buffering
	glEnable(GL_DEPTH_TEST);

	//initial values
	LMB = false;
	MMB = false;
	RMB = false;
	cameraZoom = 0.0f;
	cameraTransX = 0.0f;
	cameraTransY = -1.0f;
	cameraSpinX = 10.001f;
	cameraSpinY = -45.000f;
}

////////////////////////////////////////////////////////////////////////////////////
void draw(void)
{
	//clear frame and depth buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//reset transformation matrix to identity matrix.
	//this clears the effect of any transformations from the previous frame.
	glLoadIdentity();

	//set the camera position
	glTranslatef(cameraTransX, cameraTransY, (cameraZoom - 10.0f));
	glRotatef(cameraSpinX, 1.0f, 0.0f, 0.0f );
	glRotatef(cameraSpinY, 0.0f, 1.0f, 0.0f );

	glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);

////////////////////////////////////////////////////////////////////////////////////
//DRAW AXES
	{
		//start drawing lines
		glBegin(GL_LINES);

		//x-axis in red
		glColor3f(1, 0, 0);
		glVertex3f(0, 0, 0);
		glVertex3f(1, 0, 0);

		//y-axis in green
		glColor3f(0, 1, 0);
		glVertex3f(0, 0, 0);
		glVertex3f(0, 1, 0);

		//z-axis in green
		glColor3f(0, 0, 1);
		glVertex3f(0, 0, 0);
		glVertex3f(0, 0, 1);

		//end drawing
		glEnd();
	}


////////////////////////////////////////////////////////////////////////////////////
//DRAW GRID
	{
		//start drawing lines
		glBegin(GL_LINES);

		// x-axis in red
		glColor3f(0.2f, 0.2f, 0.2f);
		
		for(int i = -10; i <= 10; ++i)
		{
			glVertex3f(i, 0, -10);
			glVertex3f(i, 0, 10);

			glVertex3f(-10, 0, i);
			glVertex3f(10, 0, i);
		}

		//	end drawing
		glEnd();
	}

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glColor3f(0.5, 0.5, 0.5);

	//this swaps around the buffers
	glutSwapBuffers();
}

////////////////////////////////////////////////////////////////////////////////////
void resize(int windowWidth, int windowHeight)
{
	//	this prevents a division by zero when minimised
	if(windowHeight == 0)
	{
		windowHeight = 1;
	}

	//	the viewport is the area in the window we want to draw to.
	//	in this case we want to draw across the whole possible area,
	//	so the region we specify for drawing is Top Left(0,0) to bottom
	//	right (width,height)
	glViewport(0, 0, windowWidth, windowHeight);

	//	We have to set up a viewing transformation to project our 3D
	//	geometry to the 2D raster image that appears on screen. In order
	//	to do this, we have to switch the matrix mode to GL_PROJECTION. This
	//	then allows us to edit the matrix controlling the projection transform	
	glMatrixMode(GL_PROJECTION);

	//	First of all, initialise the matrix to the identity matrix.
	glLoadIdentity();

	//	We then have to specify the viewing transformation (ie, orthographic, perspective etc.
	//	You can do this manually, however it's a lot easier to use the gluPerspective function.
	//	this takes the following parameters :
	//		* viewing angle
	//		* aspect ratio
	//		* near viewing plane distance
	//		* far viewing plane distance
	gluPerspective(45.0, (float)(windowWidth / windowHeight) , 1, 500 );

	//	This sets the matrix mode back to the modelview matrix stack. This
	//	essentually allows us to rotate, scale and translate our models.
	glMatrixMode(GL_MODELVIEW);
}

////////////////////////////////////////////////////////////////////////////////////
void idle(void)
{
	//this call forces the screen to be re-drawn
	glutPostRedisplay();
}

////////////////////////////////////////////////////////////////////////////////////
void keyboardButton (unsigned char key, int x, int y)
{
	switch (key)
	{
		//escape-key
		case 27:
			exit(0);
			break;

		default:
			break;
	}
}
////////////////////////////////////////////////////////////////////////////////////
void mouseButton(int button, int state, int x, int y)
{
	// use a switch statement to check which button was pressed
	// or released. 
	switch(button) 
	{
		// the left mouse button
	case GLUT_LEFT_BUTTON :
		// set flag to indiciate mouse button is/is not pressed
		if (LMB == true)
		{
			LMB = false;
		}
		else if (LMB == false)
		{
			LMB = true;
		}
		break;

		// the middle mouse button
	case GLUT_MIDDLE_BUTTON :
		// set flag to indiciate mouse button is/is not pressed
		if (MMB == true)
		{
			MMB = false;
		}
		else if (LMB == false)
		{
			MMB = true;
		}
		break;

		// the right mouse button
	case GLUT_RIGHT_BUTTON :
		// set flag to indiciate mouse button is/is not pressed
		if (RMB == true)
		{
			RMB = false;
		}
		else if (RMB == false)
		{
			RMB = true;
		}
		break;

	default:
		break;
	}

	// store the mouse position when the button is clicked
	lastPosX = x;
	lastPosY = y;
}

////////////////////////////////////////////////////////////////////////////////////
void mouseMove(int x,int y)
{
	if( MMB || LMB || RMB )
	{
		// figure out how much we moved since last time
		stepX = x - lastPosX;
		stepY = y - lastPosY; 

		// if zooming
		if(MMB == true)
		{
			cameraZoom = cameraZoom + (0.03f * stepX);
		}
		else
		// if rotating
		if(LMB == true)
		{
			cameraSpinX = cameraSpinX + (0.3f * stepY);
			cameraSpinY = cameraSpinY + (0.3f * stepX);
		}
		else
		// if tracking
		if(RMB == true)
		{
			cameraTransX  = cameraTransX + (0.02f * stepX);
			cameraTransY  = cameraTransY - (0.02f * stepY);
		}

		// store new mouse position
		lastPosX = x;
		lastPosY = y;
	}
}


////////////////////////////////////////////////////////////////////////////////////
int main(int argc,char **argv)
{
	glutInit (&argc, argv);						//	Initialise glut
	glutInitDisplayMode (GLUT_DOUBLE |			//	Use a double buffered window (could use GLUT_SINGLE)
						GLUT_DEPTH |			//	Use depth buffering in our application (z-depth)
						GLUT_RGB);				//	Use a 24bit display (could use GLUT_RGBA for 32bit)
	glutInitWindowSize (640, 480);				//	Set window size
	glutInitWindowPosition (100, 100);			//	Set window position ( 0,0 is top left)
	glutCreateWindow ("PPC-viewer");			//	Create the window & specify title

	glutDisplayFunc (draw);						//	Let glut know what function handles re-drawing
	glutReshapeFunc (resize);					//	Let glut know what function handles re-sizing
	glutIdleFunc (idle);						//	Let glut know what function handles idle processing
	glutKeyboardFunc (keyboardButton);
	glutMouseFunc (mouseButton);				//	Let glut know what function handles mouse button presses
	glutMotionFunc (mouseMove);					//	Let glut know what function handles mouse movements when a button is held

	init();										//	call our open gl initialisation code

	glutMainLoop();								//	Enter the main window processing loop
	return 0;
}
