#include <windows.h>
#include <GL/glut.h>
#include <iostream>
#include <fstream>
#include <string>
#include <vector>
#include "vector.h"
#include "color.h"
#include "animation.h"
#include "menu.h"
#include "frame.h"
#include "pixel.h"
#include "RTG.h"

////////////////////////////////////////////////////////////////////////////////////
//global variables needed for openGL
bool playback;
bool grid;
int playbackDirection;
unsigned int nPixels = 0;
long int nFrames = 0;
long int frameCount = 0;

RTG rtgObject;
vector pPosition;
animation * pAnimation;
float pSize = 1.0;
color pColor(255, 255, 255, 255);
color pWhite(255, 255, 255, 255);
int * colorArrayR;
int * colorArrayG;
int * colorArrayB;
int * colorArrayA;
float * scaleArray;
std::vector <float> polygonVertexPositionX;
std::vector <float> polygonVertexPositionY;
std::vector <float> polygonVertexPositionZ;

////////////////////////////////////////////////////////////////////////////////////
//functions needed for openGL
void getRtgPolygonVertices();
void drawRtgObject(bool transparency, vector pPosition, color pColor);
void drawSolidCube(bool transparency, float size, vector pPosition, color pColor);
void drawWireCube(bool transparency, float size, vector pPosition, color pColor);
void init();
void display(void);
void resize(int width, int height);
void idle(void);
void mouseButton(int button, int state, int x, int y);
void mouseMove(int x,int y);
void keyboardButton (unsigned char key, int x, int y);

////////////////////////////////////////////////////////////////////////////////////
int displayMode;	//0 = wireframe, 1 = solid, 2 = solid + wireframe
bool transparency;
bool antiAliasing;
int framerate;

unsigned long int currentTime;
unsigned long int elapsedTime;
unsigned long int startTime;

bool LMB;
bool MMB;
bool RMB;

int mouseLastPosX;
int mouseLastPosY;
int mouseStepX;
int mouseStepY;	

float cameraZoom;
float cameraTransX;
float cameraTransY;
float cameraSpinX;
float cameraSpinY;

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
void main(int argc,char **argv)
{
	std::string ppcPositionPath;
	std::string ppcColorPath;
	std::string ppcOpacityPath;
	std::string ppcScalePath;

	std::ifstream ppcPosition;
	std::ifstream ppcColor;
	std::ifstream ppcOpacity;
	std::ifstream ppcScale;

	bool pathFile = false;
	bool colorFile = false;
	bool opacityFile = false;
	bool scaleFile = false;

	//const unsigned int nPixels = 0;
	unsigned int frameNumber = 0;
	//const unsigned int nFrames = 0;

	color colorDefault(255, 255, 255, 255);
	float scaleDefault = 1.0;
	vector vTemp (0, 0, 0);

	menu ppcMenu;

	//////////////////////////
	std::cout << "PPC-cruncher v1.0 beta1 provided by Stefan D'hondt" << std::endl;
	std::cout << "Please direct your fanmail to \"supersteffieke@yahoo.co.uk\"" << std::endl;
	std::cout << std::endl;
	//////////////////////////
	//show the menu
	ppcMenu.showMainMenu();

	if (ppcMenu.getRtgPath().size() != 0)
	{
		rtgObject.setRtgPath(ppcMenu.getRtgPath());
		rtgObject.analyzeRtgFile();

		if (rtgObject.bRtgExist == true)
		{
			rtgObject.printRtgInfo();
			rtgObject.printObjectInfo();
			rtgObject.getVertices(false);
			rtgObject.getNormals(false);
			rtgObject.getTextureCoordinates(false);
			rtgObject.getVertexColors(false);
			rtgObject.getPolygonAttributes(false);
			getRtgPolygonVertices();
		}
	}

if (ppcMenu.getpPathFile().size() != 0)
	{
		//read .path-file
		std::string pPathTemp = ppcMenu.getpPathFile();
		char* ptrPosPath = &pPathTemp[0];
		ppcPosition.open (ptrPosPath);

		////////////////////////////////////////////////////
		//if opening the pPath-file was successfull
		if (ppcPosition.good() == true)
		{
			//read some data
			ppcPosition >> nPixels;

			colorArrayR = new int[nPixels];
			colorArrayG = new int[nPixels];
			colorArrayB = new int[nPixels];
			colorArrayA = new int[nPixels];
			scaleArray = new float[nPixels];
		}
		ppcPosition.close();
	}

	////////////////////////////////////////////////////
	//read the specified files
	//////////////////////////
	//colorFile
	if (ppcMenu.getpColorFile().size() != 0)
	{
		std::string pColorTemp = ppcMenu.getpColorFile();
		char* ptrColorPath = &pColorTemp[0];
		ppcColor.open (ptrColorPath);
		unsigned int nPixelsTemp;

		if (ppcColor.good() == true)
		{
			ppcColor >> nPixelsTemp;

			//if number of pixels in color file equals npixels of path file
			if (nPixelsTemp == nPixels)
			{
				std::cout << "Caching pColor-file...";

				for (unsigned int i = 0; i < nPixels; i++)
				{
					ppcColor >> colorArrayR[i] >> colorArrayG[i] >> colorArrayB[i] >> colorArrayA[i];
				}
				ppcColor.close();
				colorFile = true;
				std::cout << "done" << std::endl;
				std::cout << std::endl;
			}
			else
			{
				//file couldn't open
				std::cout << "Number of pixels in pColor-file is not equal to number of pixels in pPath-file." << std::endl;
				std::cout << "Pixel color defaulted to white." << std::endl;
				std::cout << std::endl;

				for (unsigned int i = 0; i < nPixels; i++)
				{
					colorArrayR[i] = 255;
					colorArrayG[i] = 255;
					colorArrayB[i] = 255;
					colorArrayA[i] = 255;
				}
			}

		}
		else
		{
			//file couldn't open
			std::cout << "Attempt to open specified pColor-file failed." << std::endl;
			std::cout << "Pixel color defaulted to white." << std::endl;
			std::cout << std::endl;

			for (unsigned int i = 0; i < nPixels; i++)
			{
				colorArrayR[i] = 255;
				colorArrayG[i] = 255;
				colorArrayB[i] = 255;
				colorArrayA[i] = 255;
			}
		}
	}
	else
	{
		//file not defined
		std::cout << "pColor-file: Undefined" << std::endl;
		std::cout << "Pixel color defaulted to white." << std::endl;
		std::cout << std::endl;

		for (unsigned int i = 0; i < nPixels; i++)
		{
			colorArrayR[i] = 255;
			colorArrayG[i] = 255;
			colorArrayB[i] = 255;
			colorArrayA[i] = 255;
		}
	}

	//////////////////////////
	//opacityFile
	if (ppcMenu.getpOpacityFile().size() != 0)
	{
		std::string pOpacityTemp = ppcMenu.getpOpacityFile();
		char* ptrOpacityPath = &pOpacityTemp[0];
		ppcOpacity.open (ptrOpacityPath);
		unsigned int nPixelsTemp;

		if (ppcOpacity.good() == true)
		{
			ppcOpacity >> nPixelsTemp;

			//if number of pixels in opacity file equals npixels of path file
			if (nPixelsTemp == nPixels)
			{
				std::cout << "Caching pOpacity-file...";

				for (unsigned int i = 0; i < nPixels; i++)
				{
					ppcOpacity >> colorArrayA[i];
				}
				ppcOpacity.close();
				opacityFile = true;
				std::cout << "done" << std::endl;
				std::cout << std::endl;
			}
			else
			{
				//file couldn't open
				std::cout << "Number of pixels in pOpacity-file is not equal to number of pixels in pPath-file." << std::endl;
				std::cout << "Pixel opacity defaulted to 255." << std::endl;
				std::cout << std::endl;

				for (unsigned int i = 0; i < nPixels; i++)
				{
					colorArrayA[i] = 255;
				}
			}

		}
		else
		{
			//file couldn't open
			std::cout << "Attempt to open specified pOpacity-file failed." << std::endl;
			std::cout << "Pixel opacity defaulted to 255." << std::endl;
			std::cout << std::endl;

			for (unsigned int i = 0; i < nPixels; i++)
			{
				colorArrayA[i] = 255;
			}
		}
	}
	else
	{
		//file not defined
		std::cout << "pOpacity-file: Undefined" << std::endl;
		std::cout << "Pixel opacity defaulted to 255." << std::endl;
		std::cout << std::endl;

		for (unsigned int i = 0; i < nPixels; i++)
		{
			colorArrayA[i] = 255;
		}
	}
	//////////////////////////
	//scaleFile
	if (ppcMenu.getpScaleFile().size() != 0)
	{
		std::string pScaleTemp = ppcMenu.getpScaleFile();
		char* ptrScalePath = &pScaleTemp[0];
		ppcScale.open (ptrScalePath);
		unsigned int nPixelsTemp;

		if (ppcScale.good() == true)
		{
			ppcScale >> nPixelsTemp;

			if (nPixelsTemp == nPixels)
			{	
				std::cout << "Caching pScale-file...";

				for (unsigned int i = 0; i < nPixels; i++)
				{
					ppcScale >> scaleArray[i];
					scaleArray[i] = (((scaleArray[i] / 255) * (ppcMenu.getpScaleMaxConstant() - ppcMenu.getpScaleMinConstant())) + ppcMenu.getpScaleMinConstant());
				}

				ppcScale.close();
				scaleFile = true;
				std::cout << "done" << std::endl;
				std::cout << std::endl;
			}
			else
			{
				//file couldn't open
				std::cout << "Number of pixels in pScale-file is not equal to number of pixels in pPath-file." << std::endl;
				std::cout << "Pixel scale defaulted to 1.0." << std::endl;
				std::cout << std::endl;

				for (unsigned int i = 0; i < nPixels; i++)
				{
					scaleArray[i] = 1.0;
				}
			}
		}
		else
		{
			//file couldn't open
			std::cout << "Attempt to open specified pScale-file failed." << std::endl;
			std::cout << "Pixel scale defaulted to 1.0." << std::endl;
			std::cout << std::endl;

			for (unsigned int i = 0; i < nPixels; i++)
			{
				scaleArray[i] = 1.0;
			}
		}
	}
	else
	{
		//file not defined
		std::cout << "pScale-file: Undefined" << std::endl;
		std::cout << "Pixel scale defaulted to 1.0" << std::endl;
		std::cout << std::endl;

		for (unsigned int i = 0; i < nPixels; i++)
		{
			scaleArray[i] = 1.0;
		}
	}

	if (ppcMenu.getpPathFile().size() != 0)
	{
		//read .path-file
		std::string pPathTemp = ppcMenu.getpPathFile();
		char* ptrPosPath = &pPathTemp[0];
		ppcPosition.open (ptrPosPath);

		////////////////////////////////////////////////////
		//if opening the pPath-file was successfull
		if (ppcPosition.good() == true)
		{
			//read some data
			ppcPosition >> nPixels;
			std::cout << "Number of pixels: " << nPixels << std::endl;
			ppcPosition >> nFrames;
			std::cout << "Number of frames: " << nFrames << std::endl;
	
			pAnimation = new animation(nFrames, nPixels);
			pAnimation->setNumberOfFrames(nFrames);

			//read through number of frames
			for (long int i = 0; i < nFrames; i++)
			{
				std::cout << "Caching frame " << i << "...";

				pAnimation->frames[i]->setNumberOfPixels(nPixels);
				ppcPosition >> frameNumber;
	
				//read through number of pixels
				for (unsigned int j = 0; j < nPixels; j++)
				{
#if DO_OUTPUT == 1
					std::cout << "pixel: " << j << std::endl;
#endif
					ppcPosition >> vTemp.x >> vTemp.y >> vTemp.z;
					pAnimation->frames[i]->pixels[j].setPosition(vTemp);

#if DO_OUTPUT == 1
					std::cout << pAnimation->frames[i]->pixels[j].getPosition() << std::endl;
#endif
				}
				std::cout << "done" << std::endl;
			}
			ppcPosition.close();

			//////////////////////////
			//do the openGL
			std::cout << "OpenGL...";

			glutInit(&argc, argv);
			glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGBA);
			glutCreateWindow("PPC-viewer");
			glutDisplayFunc(display);
			glutReshapeFunc(resize);
			glutIdleFunc(idle);
			glutMouseFunc(mouseButton);
			glutMotionFunc(mouseMove);
			glutKeyboardFunc(keyboardButton);

			init();
			glutFullScreen();
			glutMainLoop();
		
			std::cout << "PATH-file: \"" << ppcPositionPath << "\" read successfully." << std::endl;
		}
		else
		{
			//opening path-file failed
			std::cout << "Attempt to open specified pPath-file failed." << std::endl;
			std::cout << std::endl;
			//show the menu
			ppcMenu.showMainMenu();
		}
	}

	else
	{
		//if path file is undefined
		std::cout << "Attempt to open specified pScale-file failed." << std::endl;
		std::cout << std::endl;
		//show the menu
		ppcMenu.showMainMenu();
	}

	std::cout << std::endl;
	std::cout << "Thank you for using PPC-viewer. Please come again." << std::endl;
	std::cout << "Press <ENTER> to exit." << std::endl;
	std::cin.get();
}

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
void getRtgPolygonVertices()
{
	//for (unsigned short int k = 0; k < rtgObject.getNumberOfObjects(); k++)
	//{
	int nObject = 0;
		unsigned long int polygonOffset = 0;
		unsigned long int vertexOffset = 0;
		unsigned long int numberOfVertices = 0;
		unsigned long int numberOfFaces = 0;
		float vertexPosition[3];

		for (unsigned short int objectCount = 0; objectCount < nObject; objectCount++)
		{
			polygonOffset = (polygonOffset + rtgObject.getNumberOfPolygons(objectCount));
			vertexOffset = (vertexOffset + rtgObject.getNumberOfVertices(objectCount));
		}

		numberOfFaces = rtgObject.getNumberOfPolygons(nObject);
		numberOfVertices = rtgObject.getNumberOfVertices(nObject);

		for (unsigned long int i = 0; i < numberOfFaces; i++)
		{
			for (unsigned short int j = 0; j < rtgObject.aNumberOfPolygonEdges[i]; j++)
			//for (unsigned short int j = 0; j < rtgObject.aNumberOfPolygonEdges[(polygonOffset + i)]; j++)
			{
				vertexPosition[0] = rtgObject.aVertices[(vertexOffset + rtgObject.aPolygons[(polygonOffset + i)][j])][0];
				polygonVertexPositionX.push_back(vertexPosition[0]);
				std::cout << "pos: " << vertexPosition[0] << " ";
	
				vertexPosition[1] = rtgObject.aVertices[(vertexOffset + rtgObject.aPolygons[(polygonOffset + i)][j])][1];
				polygonVertexPositionY.push_back(vertexPosition[1]);
				std::cout << vertexPosition[1] << " ";

				vertexPosition[2] = rtgObject.aVertices[(vertexOffset + rtgObject.aPolygons[(polygonOffset + i)][j])][2];
				polygonVertexPositionZ.push_back(vertexPosition[2]);
				std::cout << vertexPosition[2] << std::endl;
			}
		}
	//}
}

//////////////////////////////////////////////////////////////////////////////////////
void drawRtgObject(bool transparency, vector pPosition, color pColor)
{
	if (transparency == true)
	{
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glEnable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	else
	{
		glDisable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	if (antiAliasing == true)
	{
		glEnable(GL_LINE_SMOOTH);
		glEnable(GL_POLYGON_SMOOTH);
	}
	else
	{
		glDisable(GL_LINE_SMOOTH);
		glDisable(GL_POLYGON_SMOOTH);
	}

	unsigned long int vertexCount = 0;
	unsigned long int numberOfFaces = rtgObject.getNumberOfPolygons(0);

	for (unsigned long int i = 0; i < numberOfFaces; i++)
	{
		glBegin(GL_POLYGON);
		for (unsigned short int j = 0; j < rtgObject.aNumberOfPolygonEdges[i]; j++)
		{
			float X = (polygonVertexPositionX[vertexCount] + pPosition.x);
			float Y = (polygonVertexPositionY[vertexCount] + pPosition.y);
			float Z = (polygonVertexPositionZ[vertexCount] + pPosition.z);
			glVertex3f(X, Y, Z);
			vertexCount++;
		}
		glEnd();
	}
}


////////////////////////////////////////////////////////////////////////////////////
void drawSolidCube(bool transparency, float size, vector pPosition, color pColor)
{	
	if (transparency == true)
	{
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glEnable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	else
	{
		glDisable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	if (antiAliasing == true)
	{
		glEnable(GL_LINE_SMOOTH);
		glEnable(GL_POLYGON_SMOOTH);
	}
	else
	{
		glDisable(GL_LINE_SMOOTH);
		glDisable(GL_POLYGON_SMOOTH);
	}

	//front face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
	glEnd();
	//right face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
	glEnd();
	//back face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
	glEnd();
	//left face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
	glEnd();
	//top face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
	glEnd();
	//bottom face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
	glEnd();
}

////////////////////////////////////////////////////////////////////////////////////
void drawWireCube(bool transparency, float size, vector pPosition, color pColor)
{	
	if (transparency == true)
	{
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glEnable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	else
	{
		glDisable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	if (antiAliasing == true)
	{
		glEnable(GL_LINE_SMOOTH);
		glEnable(GL_POLYGON_SMOOTH);
	}
	else
	{
		glDisable(GL_LINE_SMOOTH);
		glDisable(GL_POLYGON_SMOOTH);
	}

	//lines between top and bottom face
	glBegin(GL_LINES);
		//line 1
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
		//line2
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
		//line3
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
		//line4
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
	glEnd();
	//top face
	glBegin(GL_LINE_LOOP);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (size / 2)));
	glEnd();
	//bottom face
	glBegin(GL_LINE_LOOP);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (size / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (size / 2)));
	glEnd();
}

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
void init()
{
	//set the default background colour to grey
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	//enable depth buffering
	glEnable(GL_DEPTH_TEST);

	playback = false;
	grid = true;
	transparency = false;
	antiAliasing = false;
	displayMode = 0;
	framerate = 25;
	currentTime = 0;
	elapsedTime = 0;
	startTime = 0;
	playbackDirection = 1;	//1: to the right, -1: to the left

	//set initial defaults for houdini-style input
	LMB = false;
	MMB = false;
	RMB = false;
	cameraZoom = -12.0f;
	cameraTransX = 0.0f;
	cameraTransY = -2.0f;
	cameraSpinX = 10.000f;
	cameraSpinY = -45.000f;

	//start timer
	currentTime = GetTickCount();
	startTime = GetTickCount();
}

////////////////////////////////////////////////////////////////////////////////////
void display(void)
{
	//this clears the framebuffer and the depth buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//reset the transformation matrix to the identity matrix. This
	//clears the effect of any transformations from the previous frame.
	glLoadIdentity();

	//set the camera position
	glTranslatef(cameraTransX, cameraTransY, cameraZoom-10.0f);
	glRotatef(cameraSpinX, 1.0f, 0.0f, 0.0f);
	glRotatef(cameraSpinY, 0.0f, 1.0f, 0.0f);

	if (grid == true)
	{
		glDisable(GL_LIGHTING);
		glDisable(GL_LIGHT0);

		if (antiAliasing == true)
		{
			glEnable(GL_LINE_SMOOTH);
		}
		else
		{
			glDisable(GL_LINE_SMOOTH);
		}
		//draw grid
		{
			glLineWidth(1.0f);
			//start drawing lines
			glBegin(GL_LINES);
	
			glColor3f(0.2f,0.2f,0.2f);
		
			for(int i = -10; i <= 10; ++i)
			{
				glVertex3f(i,0,-10);
				glVertex3f(i,0,10);

				glVertex3f(-10,0,i);
				glVertex3f(10,0,i);
			}

			//end drawing
			glEnd();
		}

		//draw axes
		{
			glLineWidth(2.0f);
			//start drawing lines
			glBegin(GL_LINES);

			//x-axis in red
			glColor3f(1,0,0);
			glVertex3f(0,0,0);
			glVertex3f(1,0,0);

			//y-axis in green
			glColor3f(0,1,0);
			glVertex3f(0,0,0);
			glVertex3f(0,1,0);

			//z-axis in green
			glColor3f(0,0,1);
			glVertex3f(0,0,0);
			glVertex3f(0,0,1);

			//end drawing
			glEnd();
		}
	}

	//disable antiAliasing after drawing grid and origin axes
	glDisable(GL_LINE_SMOOTH);

	//while plaing back
	if (playback == true)
	{
		currentTime = GetTickCount();
		elapsedTime = currentTime - startTime;

		if (static_cast <float> (elapsedTime) >= (1000 / framerate))
		{
			std::cout << "framerate: " <<  (1000 / elapsedTime) << std::endl;
			startTime = GetTickCount();

			//set line width
			glLineWidth(1.25f);

			for (unsigned int i = 0; i < nPixels; i++)
			{
				pPosition.x = pAnimation->frames[frameCount]->pixels[i].getPosition().x;
				pPosition.y = pAnimation->frames[frameCount]->pixels[i].getPosition().y;
				pPosition.z = pAnimation->frames[frameCount]->pixels[i].getPosition().z;
				pColor.red = colorArrayR[i];
				pColor.green = colorArrayG[i];
				pColor.blue = colorArrayB[i];
				pColor.alpha = colorArrayA[i];
				pSize = scaleArray[i];

				glShadeModel(GL_SMOOTH);
				//if wireframe mode
				if (displayMode == 0)
				{
					if (rtgObject.bRtgExist == true)
					{
						drawRtgObject(transparency, pPosition, pColor);
					}
					else
					{
						drawWireCube(transparency, pSize, pPosition, pColor);
					}
				}
				//if solid shade mode
				else if(displayMode == 1)
				{
					if (rtgObject.bRtgExist == true)
					{
						drawRtgObject(transparency, pPosition, pColor);
					}
					else
					{
						drawSolidCube(transparency, pSize, pPosition, pColor);
					}
				}
				//if solid + wireframe mode
				else if(displayMode == 2)
				{
					if (rtgObject.bRtgExist == true)
					{
						drawRtgObject(transparency, pPosition, pColor);
					}
					else
					{
						drawSolidCube(transparency, pSize, pPosition, pColor);
						drawWireCube(transparency, pSize, pPosition, pWhite);
					}
				}
			}

			if (playbackDirection == 1)
			{
				frameCount++;
			}
			else if (playbackDirection == -1)
			{
				frameCount--;
			}

			if (frameCount == nFrames)
			{
				frameCount = 0;
			}
			if(frameCount == -1)
			{
				frameCount = (nFrames - 1);
			}

			glFlush();
			glutSwapBuffers();
		}

	}
	//if playback is stopped
	else
	{
		for (unsigned int i = 0; i < nPixels; i++)
		{
			pPosition.x = pAnimation->frames[frameCount]->pixels[i].getPosition().x;
			pPosition.y = pAnimation->frames[frameCount]->pixels[i].getPosition().y;
			pPosition.z = pAnimation->frames[frameCount]->pixels[i].getPosition().z;
			pColor.red = colorArrayR[i];
			pColor.green = colorArrayG[i];
			pColor.blue = colorArrayB[i];
			pColor.alpha = colorArrayA[i];
			pSize = scaleArray[i];

			//set line width
			glLineWidth(1.25f);

			glShadeModel(GL_SMOOTH);
			//if wireframe mode
			if (displayMode == 0)
			{
				if (rtgObject.bRtgExist == true)
				{
					drawRtgObject(transparency, pPosition, pColor);
				}
				else
				{
					drawWireCube(transparency, pSize, pPosition, pColor);
				}
			}
			//if solid shade mode
			else if(displayMode == 1)
			{
				if (rtgObject.bRtgExist == true)
				{
					drawRtgObject(transparency, pPosition, pColor);
				}
				else
				{
					drawSolidCube(transparency, pSize, pPosition, pColor);
				}
			}
			//if solid + wireframe mode
			else if(displayMode == 2)
			{
				if (rtgObject.bRtgExist == true)
				{
					drawRtgObject(transparency, pPosition, pColor);
				}
				else
				{
					drawSolidCube(transparency, pSize, pPosition, pColor);
					drawWireCube(transparency, pSize, pPosition, pWhite);
				}
			}
		}
		glFlush();
		glutSwapBuffers();
	}

	//glFlush();
	//this swaps around the buffers
	//glutSwapBuffers();
}

////////////////////////////////////////////////////////////////////////////////////
void resize(int width, int height)
{
	//this prevents a division by zero when minimised
	if(height == 0)
	{
		height = 1;
	}

	//the viewport is the area in the window we want to draw to.
	//in this case we want to draw across the whole possible area,
	//so the region we specify for drawing is Top Left(0,0) to bottom
	//right (width,height)
	glViewport(0, 0, width, height);

	//we have to set up a viewing transformation to project our 3D
	//geometry to the 2D raster image that appears on screen. In order
	//to do this, we have to switch the matrix mode to GL_PROJECTION. This
	//then allows us to edit the matrix controlling the projection transform	
	glMatrixMode(GL_PROJECTION);

	//first of all, initialise the matrix to the identity matrix.
	glLoadIdentity();

	//we then have to specify the viewing transformation (ie, orthographic, perspective etc.
	//You can do this manually, however it's a lot easier to use the gluPerspective function.
	//this takes the following parameters :
	//* viewing angle
	//* aspect ratio
	//* near viewing plane distance
	//* far viewing plane distance
	gluPerspective(45.0, (float)width/height , 1, 500);

	//this sets the matrix mode back to the modelview matrix stack. This
	//essentually allows us to rotate, scale and translate our models.
	glMatrixMode(GL_MODELVIEW);
}

////////////////////////////////////////////////////////////////////////////////////
void idle(void)
{
	//this call forces the screen to be re-drawn
	glutPostRedisplay();
}

////////////////////////////////////////////////////////////////////////////////////
void mouseButton(int button, int state, int x, int y)
{ 
	switch(button) 
	{
	//the left mouse button
	case GLUT_LEFT_BUTTON :
		if(LMB == true)
		{
			LMB = false;
		}
		else if(LMB == false)
		{
			LMB = true;
		}
		break;

	//the middle mouse button
	case GLUT_MIDDLE_BUTTON :
		if(MMB == true)
		{
			MMB = false;
		}
		else if(MMB == false)
		{
			MMB = true;
		}
		break;

	//the right mouse button
	case GLUT_RIGHT_BUTTON :
		if(RMB == true)
		{
			RMB = false;
		}
		else if(RMB == false)
		{
			RMB = true;
		}
		break;

	default:
		break;
	}

	//store the mouse position when the button is clicked
	mouseLastPosX = x;
	mouseLastPosY = y;
}

////////////////////////////////////////////////////////////////////////////////////
void mouseMove(int x,int y)
{
	if( MMB || LMB || RMB )
	{
		// figure out how much we moved since last time
		mouseStepX = x - mouseLastPosX;
		mouseStepY = y - mouseLastPosY; 

		// if rotating
		if(LMB == true)
		{
			cameraSpinX += 0.5f * mouseStepY;
			cameraSpinY += 0.5f * mouseStepX;
		}
		else
		// if zooming
		if(MMB == true)
		{
			cameraZoom += 0.1f * mouseStepX;
		}
		else
		// if tracking
		if(RMB == true)
		{
			cameraTransX  += 0.05f*mouseStepX;
			cameraTransY  -= 0.05f*mouseStepY;
		}

		// store new mouse position
		mouseLastPosX = x;
		mouseLastPosY = y;
	}
}

////////////////////////////////////////////////////////////////////////////////////
void keyboardButton (unsigned char key, int x, int y)
{
	switch (key)
	{
		//escape-key
		//exit
		case 27:
			std::cout << "openGL finished" << std::endl;
			exit(0);
			break;

		//SPACE-bar
		//playback on/off
		case 32:
			if (playback == true)
			{
				playback = false;
			}
			else
			{
				playback = true;
			}
			break;

		//antiAliasing on/off
		case 'a':
		case 'A':
			if (antiAliasing == true)
			{
				antiAliasing = false;
			}
			else
			{
				antiAliasing = true;
			}
			break;

		//cycle through display modes
		case 'd':
		case 'D':
			if (displayMode == 0)
			{
				displayMode = 1;
			}
			else if (displayMode == 1)
			{
				displayMode = 2;
			}
			else if (displayMode == 2)
			{
				displayMode = 0;
			}
			break;

		//reset camera position
		case 'f':
		case 'F':
			cameraZoom = -12.0f;
			cameraTransX = 0.0f;
			cameraTransY = -2.0f;
			cameraSpinX = 10.000f;
			cameraSpinY = -45.000f;
			break;

		//grid on/off
		case 'g':
		case 'G':
			if (grid == true)
			{
				grid = false;
			}
			else
			{
				grid = true;
			}
			break;

		//transparency on/off
		case 't':
		case 'T':
			if (transparency == true)
			{
				transparency = false;
			}
			else
			{
				transparency = true;
			}
			break;

		//playing forward
		case '.':
		case '>':
			if (playback == true)
			{
				playbackDirection = 1;
			}

			if (playback == false)
			{
				frameCount++;

				if (frameCount == nFrames)
				{
					frameCount = 0;
				}
			}
			break;

		//playing backward
		case ',':
		case '<':
			if (playback == true)
			{
			playbackDirection = -1;
			}

			if (playback == false)
			{
				frameCount--;

				if (frameCount == -1)
				{
					frameCount = (nFrames - 1);
				}
			}
			break;

		default:
			break;
	}
}