#include <windows.h>
#include <GL/glut.h>
#include <iostream>
#include <fstream>
#include <string>
#include "vector.h"
#include "ltga.h"
#include "color.h"
#include "animation.h"
#include "menu.h"
#include "keyframe.h"
#include "pixel.h"
#include "wrapper.h"

////////////////////////////////////////////////////////////////////////////////////
//functions needed for openGL
void getRtgPolygonVertices();
void drawSolidCube(unsigned long int pCount, bool transparency, float size, vector pPosition, color pColor);
void drawWireCube(unsigned long int pCount, bool transparency, float size, vector pPosition, color pColor);
void init();
void display(void);
void resize(int width, int height);
void idle(void);
void mouseButton(int button, int state, int x, int y);
void mouseMove(int x,int y);
void keyboardButton (unsigned char key, int x, int y);

/////////////////////////////////////////////////////////////////
//openGL variables
int playbackDirection = 1;		//forward
bool playback;
bool grid;
long int frameCount = 0;
long int keyframeCount = 0;
long int interpolationframeCount = 0;
unsigned long int totalNumberOfFrames = 0;

float ** mDisplacement;
float * scaleArray;
int ** colorR;
int ** colorG;
int ** colorB;
int ** colorA;
color pColor(255, 255, 255, 255);
color pWhite(255, 255, 255, 255);

int displayMode;	//0 = wireframe, 1 = solid, 2 = solid + wireframe
bool transparency;
bool antiAliasing;
int framerate;

unsigned long int currentTime;
unsigned long int elapsedTime;
unsigned long int startTime;

bool LMB;
bool MMB;
bool RMB;

int mouseLastPosX;
int mouseLastPosY;
int mouseStepX;
int mouseStepY;	

float cameraZoom;
float cameraTransX;
float cameraTransY;
float cameraSpinX;
float cameraSpinY;

/////////////////////////////////////////////////////////////////
menu iBlenderMenu;
unsigned char displacementMode;
float maxDisplacement = 0.0;
float minDisplacement = 0.0;
animation * pAnimation;
float * heightDisplacement;
float calculateDisplacement(unsigned char displacementMode, unsigned long int pixelNumber, color pixelColor);

/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
void main(int argc,char **argv)
{
	std::cout << "iBlender v1.0" << std::endl;
	std::cout << "NCCA 2003" << std::endl;
	std::cout << std::endl;
	
	iBlenderMenu.showMenuItem1();
	iBlenderMenu.getNumberOfImages();

	minDisplacement = iBlenderMenu.getMinDisplacement();
	maxDisplacement = iBlenderMenu.getMaxDisplacement();
	displacementMode = 'c';
	heightDisplacement = new float[iBlenderMenu.mImage[0].getNumberOfPixels()];
	pAnimation = new animation(iBlenderMenu.getNumberOfImages(), iBlenderMenu.mImage[0].getNumberOfPixels());

	for (unsigned short int i = 0; i < iBlenderMenu.getNumberOfImages(); i++)
	{
		for (unsigned long int j = 0; j < iBlenderMenu.mImage[0].getNumberOfPixels(); j++)
		{
			pAnimation->keyframes[i]->pixels[j].rgba.red = static_cast <int> (iBlenderMenu.mImage[i].pixels[j].rgba.red);
			pAnimation->keyframes[i]->pixels[j].rgba.green = static_cast <int> (iBlenderMenu.mImage[i].pixels[j].rgba.green);
			pAnimation->keyframes[i]->pixels[j].rgba.blue = static_cast <int> (iBlenderMenu.mImage[i].pixels[j].rgba.blue);
			pAnimation->keyframes[i]->pixels[j].rgba.alpha = static_cast <int> (iBlenderMenu.mImage[i].pixels[j].rgba.alpha);
		}
		pAnimation->keyframes[i]->setnInterpolationFrames(iBlenderMenu.getInterpolationFrames(i));
		totalNumberOfFrames = totalNumberOfFrames + iBlenderMenu.getInterpolationFrames(i);
	}

	totalNumberOfFrames = totalNumberOfFrames + iBlenderMenu.getNumberOfImages();

	colorR = new int*[totalNumberOfFrames];
	colorG = new int*[totalNumberOfFrames];
	colorB = new int*[totalNumberOfFrames];
	colorA = new int*[totalNumberOfFrames];
	mDisplacement = new float*[totalNumberOfFrames];

	for (unsigned int i = 0; i < totalNumberOfFrames; i++)
	{
		colorR[i] = new int[iBlenderMenu.mImage[0].getNumberOfPixels()];
		colorG[i] = new int[iBlenderMenu.mImage[0].getNumberOfPixels()];
		colorB[i] = new int[iBlenderMenu.mImage[0].getNumberOfPixels()];
		colorA[i] = new int[iBlenderMenu.mImage[0].getNumberOfPixels()];
		mDisplacement[i] = new float[iBlenderMenu.mImage[0].getNumberOfPixels()];
	}

	//step through # of key images
	for (int i = 0; i < iBlenderMenu.getNumberOfImages(); i++)
	{
		std::cout << "---------------------------------------" << std::endl;
		std::cout << "keyframe: " << i << std::endl;

		//step through # of pixels
		for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
		{
			//calculate current color on a per pixel basis
			int red = pAnimation->keyframes[i]->pixels[pCount].rgba.red;
			int green = pAnimation->keyframes[i]->pixels[pCount].rgba.green;
			int blue = pAnimation->keyframes[i]->pixels[pCount].rgba.blue;
			int alpha = pAnimation->keyframes[i]->pixels[pCount].rgba.alpha;

			heightDisplacement[pCount] = calculateDisplacement(displacementMode, pCount, pAnimation->keyframes[i]->pixels[pCount].rgba);
		}

			//step current image's interpolation frames
			for (int j = 0; j < pAnimation->keyframes[i]->getnInterpolationFrames(); j++)
			{
				//step through # of pixels
				for (unsigned int k = 0; k < iBlenderMenu.mImage[0].getNumberOfPixels(); k++)
				{
					color cTemp;
								
					//if not at last keyframe
					if (i != (iBlenderMenu.getNumberOfImages() - 1))
					{
						//calculate current color on a per pixel basis
						int redDiff = (static_cast <int>(pAnimation->keyframes[(i + 1)]->pixels[k].rgba.red) - static_cast <int>(pAnimation->keyframes[i]->pixels[k].rgba.red));
						cTemp.red = ((redDiff * (j + 1)) / pAnimation->keyframes[i]->getnInterpolationFrames()) + pAnimation->keyframes[i]->pixels[k].rgba.red;
						colorR[frameCount][k] = cTemp.red;

						int greenDiff = (static_cast <int>(pAnimation->keyframes[(i + 1)]->pixels[k].rgba.green) - static_cast <int>(pAnimation->keyframes[i]->pixels[k].rgba.green));
						cTemp.green = ((greenDiff * (j + 1)) / pAnimation->keyframes[i]->getnInterpolationFrames()) + pAnimation->keyframes[i]->pixels[k].rgba.green;
						colorG[frameCount][k] = cTemp.green;

						int blueDiff = (static_cast <int>(pAnimation->keyframes[(i + 1)]->pixels[k].rgba.blue) - static_cast <int>(pAnimation->keyframes[i]->pixels[k].rgba.blue));
						cTemp.blue = ((blueDiff * (j + 1)) / pAnimation->keyframes[i]->getnInterpolationFrames()) + pAnimation->keyframes[i]->pixels[k].rgba.blue;
						colorB[frameCount][k] = cTemp.blue;

						int alphaDiff = (static_cast <int>(pAnimation->keyframes[(i + 1)]->pixels[k].rgba.alpha) - static_cast <int>(pAnimation->keyframes[i]->pixels[k].rgba.alpha));
						cTemp.alpha = ((alphaDiff * (j + 1)) / pAnimation->keyframes[i]->getnInterpolationFrames()) + pAnimation->keyframes[i]->pixels[k].rgba.alpha;
						colorA[frameCount][k] = cTemp.alpha;
					}
					//if at last keyframe
					if (i == (iBlenderMenu.getNumberOfImages() - 1))
					{
						//calculate current color on a per pixel basis
						int redDiff = (static_cast <int>(pAnimation->keyframes[0]->pixels[k].rgba.red) - static_cast <int>(pAnimation->keyframes[i]->pixels[k].rgba.red));
						cTemp.red = ((redDiff * (j + 1))/ pAnimation->keyframes[i]->getnInterpolationFrames()) + pAnimation->keyframes[i]->pixels[k].rgba.red;
						colorR[frameCount][k] = cTemp.red;

						int greenDiff = (static_cast <int>(pAnimation->keyframes[0]->pixels[k].rgba.green) - static_cast <int>(pAnimation->keyframes[i]->pixels[k].rgba.green));
						cTemp.green = ((greenDiff * (j + 1))/ pAnimation->keyframes[i]->getnInterpolationFrames()) + pAnimation->keyframes[i]->pixels[k].rgba.green;
						colorG[frameCount][k] = cTemp.green;

						int blueDiff = (static_cast <int>(pAnimation->keyframes[0]->pixels[k].rgba.blue) - static_cast <int>(pAnimation->keyframes[i]->pixels[k].rgba.blue));
						cTemp.blue = ((blueDiff * (j + 1))/ pAnimation->keyframes[i]->getnInterpolationFrames()) + pAnimation->keyframes[i]->pixels[k].rgba.blue;
						colorB[frameCount][k] = cTemp.blue;

						int alphaDiff = (static_cast <int>(pAnimation->keyframes[0]->pixels[k].rgba.alpha) - static_cast <int>(pAnimation->keyframes[i]->pixels[k].rgba.alpha));
						cTemp.alpha = ((alphaDiff * (j + 1))/ pAnimation->keyframes[i]->getnInterpolationFrames()) + pAnimation->keyframes[i]->pixels[k].rgba.alpha;
						colorA[frameCount][k] = cTemp.alpha;
					}
					heightDisplacement[k] = calculateDisplacement(displacementMode, k, cTemp);
					mDisplacement[frameCount][k] = heightDisplacement[k];
				}
				frameCount++;
			}
	}

	frameCount = 0;

	//////////////////////////
	//do the openGL
	std::cout << "OpenGL...";

	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGBA);
	glutCreateWindow("PPC-viewer");
	glutDisplayFunc(display);
	glutReshapeFunc(resize);
	glutIdleFunc(idle);
	glutMouseFunc(mouseButton);
	glutMotionFunc(mouseMove);
	glutKeyboardFunc(keyboardButton);

	init();
	glutFullScreen();
	glutMainLoop();

	std::cout << "Thank you for using iBlender. Please come again." << std::endl;
	std::cout << "Press <ENTER> to exit...";
	std::cin.get();
}

/////////////////////////////////////////////////////////////////
float calculateDisplacement(unsigned char displacementMode, unsigned long int pixelNumber, color pixelColor)
{
	float displacementDifference = (maxDisplacement - minDisplacement);
	float colorAvg = ((pixelColor.red + pixelColor.green + pixelColor.blue) / 3.0);
	int red = static_cast <int> (pixelColor.red);
	int green = static_cast <int> (pixelColor.green);
	int blue = static_cast <int> (pixelColor.blue);
	int alpha = static_cast <int> (pixelColor.alpha);

	switch (displacementMode)
	{
	case 'a':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / alpha)) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	case 'A':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / (255 - alpha))) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	case 'b':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / blue)) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	case 'B':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / (255 - blue))) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	case 'c':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / colorAvg)) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	case 'C':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / (255 - colorAvg))) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	case 'g':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / green)) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	case 'G':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / (255 - green))) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	case 'n':
	case 'N':
		heightDisplacement[pixelNumber] = 0;
		return heightDisplacement[pixelNumber];
		break;

	case 'r':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / red)) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	case 'R':
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / (255 - red))) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;

	default:
		heightDisplacement[pixelNumber] = ((displacementDifference / (255 / (255 - colorAvg))) + minDisplacement);
		return heightDisplacement[pixelNumber];
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////////
void drawSolidCube(unsigned long int pCount, bool transparency, float size, vector pPosition, color pColor)
{	
	if (transparency == true)
	{
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glEnable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	else
	{
		glDisable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	if (antiAliasing == true)
	{
		glEnable(GL_LINE_SMOOTH);
		glEnable(GL_POLYGON_SMOOTH);
	}
	else
	{
		glDisable(GL_LINE_SMOOTH);
		glDisable(GL_POLYGON_SMOOTH);
	}

	//front face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
	glEnd();
	//right face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
	glEnd();
	//back face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
	glEnd();
	//left face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
	glEnd();
	//top face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
	glEnd();
	//bottom face
	glBegin(GL_POLYGON);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
	glEnd();
}

////////////////////////////////////////////////////////////////////////////////////
void drawWireCube(unsigned long int pCount, bool transparency, float size, vector pPosition, color pColor)
{	
	if (transparency == true)
	{
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glEnable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	else
	{
		glDisable(GL_BLEND);
		glColor4ub(pColor.red, pColor.green, pColor.blue, pColor.alpha);
	}
	if (antiAliasing == true)
	{
		glEnable(GL_LINE_SMOOTH);
		glEnable(GL_POLYGON_SMOOTH);
	}
	else
	{
		glDisable(GL_LINE_SMOOTH);
		glDisable(GL_POLYGON_SMOOTH);
	}

	//lines between top and bottom face
	glBegin(GL_LINES);
		//line 1
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		//line2
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		//line3
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		//line4
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
	glEnd();
	//top face
	glBegin(GL_LINE_LOOP);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y + (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
	glEnd();
	//bottom face
	glBegin(GL_LINE_LOOP);
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z + (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x + (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
		glVertex3f((pPosition.x - (size / 2)), (pPosition.y - (size / 2)), (pPosition.z - (mDisplacement[frameCount][pCount] / 2)));
	glEnd();
}

////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
void init()
{
	//set the default background colour to grey
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	//enable depth buffering
	glEnable(GL_DEPTH_TEST);

	playback = false;
	grid = true;
	transparency = false;
	antiAliasing = false;
	displayMode = 0;
	framerate = 25;
	currentTime = 0;
	elapsedTime = 0;
	startTime = 0;
	playbackDirection = 1;	//1: to the right, -1: to the left

	//set initial defaults for houdini-style input
	LMB = false;
	MMB = false;
	RMB = false;
	cameraZoom = -12.0f;
	cameraTransX = 0.0f;
	cameraTransY = -2.0f;
	cameraSpinX = 10.000f;
	cameraSpinY = -45.000f;

	//start timer
	currentTime = GetTickCount();
	startTime = GetTickCount();
}

////////////////////////////////////////////////////////////////////////////////////
void display(void)
{
	//this clears the framebuffer and the depth buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//reset the transformation matrix to the identity matrix. This
	//clears the effect of any transformations from the previous frame.
	glLoadIdentity();

	//set the camera position
	glTranslatef(cameraTransX, cameraTransY, cameraZoom-10.0f);
	glRotatef(cameraSpinX, 1.0f, 0.0f, 0.0f);
	glRotatef(cameraSpinY, 0.0f, 1.0f, 0.0f);

	if (grid == true)
	{
		glDisable(GL_LIGHTING);
		glDisable(GL_LIGHT0);

		if (antiAliasing == true)
		{
			glEnable(GL_LINE_SMOOTH);
		}
		else
		{
			glDisable(GL_LINE_SMOOTH);
		}
		//draw grid
		{
			glLineWidth(1.0f);
			//start drawing lines
			glBegin(GL_LINES);
	
			glColor3f(0.2f,0.2f,0.2f);
		
			for(int i = -10; i <= 10; ++i)
			{
				glVertex3f(i,0,-10);
				glVertex3f(i,0,10);

				glVertex3f(-10,0,i);
				glVertex3f(10,0,i);
			}

			//end drawing
			glEnd();
		}

		//draw axes
		{
			glLineWidth(2.0f);
			//start drawing lines
			glBegin(GL_LINES);

			//x-axis in red
			glColor3f(1,0,0);
			glVertex3f(0,0,0);
			glVertex3f(1,0,0);

			//y-axis in green
			glColor3f(0,1,0);
			glVertex3f(0,0,0);
			glVertex3f(0,1,0);

			//z-axis in green
			glColor3f(0,0,1);
			glVertex3f(0,0,0);
			glVertex3f(0,0,1);

			//end drawing
			glEnd();
		}
	}

	//disable antiAliasing after drawing grid and origin axes
	glDisable(GL_LINE_SMOOTH);

	//while plaing back
	if (playback == true)
	{
		currentTime = GetTickCount();
		elapsedTime = currentTime - startTime;

		if (static_cast <float> (elapsedTime) >= (1000 / framerate))
		{
			std::cout << "framerate: " <<  (1000 / elapsedTime) << std::endl;
			startTime = GetTickCount();

			//set line width
			glLineWidth(1.25f);
			glShadeModel(GL_SMOOTH);

			if (displayMode == 0)
			{
				//step through # of pixels
				for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
				{
					color cTemp(colorR[frameCount][pCount], colorG[frameCount][pCount], colorB[frameCount][pCount], colorA[frameCount][pCount]);
					drawWireCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), cTemp);
				}
			}
			//if solid shade mode
			else if(displayMode == 1)
			{
				//step through # of pixels
				for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
				{
					color cTemp(colorR[frameCount][pCount], colorG[frameCount][pCount], colorB[frameCount][pCount], colorA[frameCount][pCount]);
					drawSolidCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), cTemp);
				}
			}
			else if(displayMode == 2)
			{
				//step through # of pixels
				for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
				{
					color cTemp(colorR[frameCount][pCount], colorG[frameCount][pCount], colorB[frameCount][pCount], colorA[frameCount][pCount]);
					drawSolidCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), cTemp);
					drawWireCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), pWhite);
				}
			}

			frameCount++;
			interpolationframeCount++;
			
			if (interpolationframeCount == pAnimation->keyframes[keyframeCount]->getnInterpolationFrames())
			{
				interpolationframeCount = 0;
				keyframeCount++;

				if (keyframeCount == pAnimation->getNumberOfKeyFrames())
				{
					frameCount = 0;
					keyframeCount = 0;
				}
			}
		}

		else
		{
			//set line width
			glLineWidth(1.25f);
			glShadeModel(GL_SMOOTH);

			if (displayMode == 0)
			{
				//step through # of pixels
				for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
				{
					color cTemp(colorR[frameCount][pCount], colorG[frameCount][pCount], colorB[frameCount][pCount], colorA[frameCount][pCount]);
					drawWireCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), cTemp);
				}
			}
			//if solid shade mode
			else if(displayMode == 1)
			{
				//step through # of pixels
				for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
				{
					color cTemp(colorR[frameCount][pCount], colorG[frameCount][pCount], colorB[frameCount][pCount], colorA[frameCount][pCount]);
					drawSolidCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), cTemp);
				}
			}
			else if(displayMode == 2)
			{
				//step through # of pixels
				for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
				{
					color cTemp(colorR[frameCount][pCount], colorG[frameCount][pCount], colorB[frameCount][pCount], colorA[frameCount][pCount]);
					drawSolidCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), cTemp);
					drawWireCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), pWhite);
				}
			}
		}

		glFlush();
		glutSwapBuffers();
		}

	//if playback is stopped
	else
	{
			//set line width
			glLineWidth(1.25f);
			glShadeModel(GL_SMOOTH);
			//if wireframe mode
			if (displayMode == 0)
			{
				//step through # of pixels
				for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
				{
					color cTemp(colorR[frameCount][pCount], colorG[frameCount][pCount], colorB[frameCount][pCount], colorA[frameCount][pCount]);
					drawWireCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), cTemp);
				}
			}
			//if solid shade mode
			else if(displayMode == 1)
			{
				//step through # of pixels
				for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
				{
					color cTemp(colorR[frameCount][pCount], colorG[frameCount][pCount], colorB[frameCount][pCount], colorA[frameCount][pCount]);
					drawSolidCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), cTemp);
				}
			}
			else if(displayMode == 2)
			{
				//step through # of pixels
				for (unsigned int pCount = 0; pCount < iBlenderMenu.mImage[0].getNumberOfPixels(); pCount++)
				{
					color cTemp(colorR[frameCount][pCount], colorG[frameCount][pCount], colorB[frameCount][pCount], colorA[frameCount][pCount]);
					drawSolidCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), cTemp);
					drawWireCube(pCount, transparency, 1, iBlenderMenu.mImage[keyframeCount].pixels[pCount].getPosition(), pWhite);
				}
			}
		glFlush();
		glutSwapBuffers();

	}
}

////////////////////////////////////////////////////////////////////////////////////
void resize(int width, int height)
{
	//this prevents a division by zero when minimised
	if(height == 0)
	{
		height = 1;
	}

	//the viewport is the area in the window we want to draw to.
	//in this case we want to draw across the whole possible area,
	//so the region we specify for drawing is Top Left(0,0) to bottom
	//right (width,height)
	glViewport(0, 0, width, height);

	//we have to set up a viewing transformation to project our 3D
	//geometry to the 2D raster image that appears on screen. In order
	//to do this, we have to switch the matrix mode to GL_PROJECTION. This
	//then allows us to edit the matrix controlling the projection transform	
	glMatrixMode(GL_PROJECTION);

	//first of all, initialise the matrix to the identity matrix.
	glLoadIdentity();

	//we then have to specify the viewing transformation (ie, orthographic, perspective etc.
	//You can do this manually, however it's a lot easier to use the gluPerspective function.
	//this takes the following parameters :
	//* viewing angle
	//* aspect ratio
	//* near viewing plane distance
	//* far viewing plane distance
	gluPerspective(45.0, (float)width/height , 1, 500);

	//this sets the matrix mode back to the modelview matrix stack. This
	//essentually allows us to rotate, scale and translate our models.
	glMatrixMode(GL_MODELVIEW);
}

////////////////////////////////////////////////////////////////////////////////////
void idle(void)
{
	//this call forces the screen to be re-drawn
	glutPostRedisplay();
}

////////////////////////////////////////////////////////////////////////////////////
void mouseButton(int button, int state, int x, int y)
{ 
	switch(button) 
	{
	//the left mouse button
	case GLUT_LEFT_BUTTON :
		if(LMB == true)
		{
			LMB = false;
		}
		else if(LMB == false)
		{
			LMB = true;
		}
		break;

	//the middle mouse button
	case GLUT_MIDDLE_BUTTON :
		if(MMB == true)
		{
			MMB = false;
		}
		else if(MMB == false)
		{
			MMB = true;
		}
		break;

	//the right mouse button
	case GLUT_RIGHT_BUTTON :
		if(RMB == true)
		{
			RMB = false;
		}
		else if(RMB == false)
		{
			RMB = true;
		}
		break;

	default:
		break;
	}

	//store the mouse position when the button is clicked
	mouseLastPosX = x;
	mouseLastPosY = y;
}

////////////////////////////////////////////////////////////////////////////////////
void mouseMove(int x,int y)
{
	if( MMB || LMB || RMB )
	{
		// figure out how much we moved since last time
		mouseStepX = x - mouseLastPosX;
		mouseStepY = y - mouseLastPosY; 

		// if rotating
		if(LMB == true)
		{
			cameraSpinX += 0.5f * mouseStepY;
			cameraSpinY += 0.5f * mouseStepX;
		}
		else
		// if zooming
		if(MMB == true)
		{
			cameraZoom += 0.1f * mouseStepX;
		}
		else
		// if tracking
		if(RMB == true)
		{
			cameraTransX  += 0.05f*mouseStepX;
			cameraTransY  -= 0.05f*mouseStepY;
		}

		// store new mouse position
		mouseLastPosX = x;
		mouseLastPosY = y;
	}
}

////////////////////////////////////////////////////////////////////////////////////
void keyboardButton (unsigned char key, int x, int y)
{
	switch (key)
	{
		//escape-key
		//exit
		case 27:
			std::cout << "openGL finished" << std::endl;
			exit(0);
			break;

		//SPACE-bar
		//playback on/off
		case 32:
			if (playback == true)
			{
				playback = false;
			}
			else
			{
				playback = true;
			}
			break;

		//cycle through display modes
		case 'd':
		case 'D':
			if (displayMode == 0)
			{
				displayMode = 1;
			}
			else if (displayMode == 1)
			{
				displayMode = 2;
			}
			else if (displayMode == 2)
			{
				displayMode = 0;
			}
			break;

		//reset camera position
		case 'f':
		case 'F':
			cameraZoom = -12.0f;
			cameraTransX = 0.0f;
			cameraTransY = -2.0f;
			cameraSpinX = 10.000f;
			cameraSpinY = -45.000f;
			break;

		//grid on/off
		case 'g':
		case 'G':
			if (grid == true)
			{
				grid = false;
			}
			else
			{
				grid = true;
			}
			break;

		//transparency on/off
		case 't':
		case 'T':
			if (transparency == true)
			{
				transparency = false;
			}
			else
			{
				transparency = true;
			}
			break;

		default:
			break;
	}
}