#pragma once
#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include "menu.h"
#include "wrapper.h"

/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////
menu::menu()
{
	menuItem = ' ';
	iNumberOfImages = 0;
	bConfigurationFile = false;
}

/////////////////////////////////////////////////////////////////
menu::~menu()
{
}

/////////////////////////////////////////////////////////////////
void menu::setConfigurationFile(std::string configurationPath)
{
	strConfigurationFile = configurationPath;
}

/////////////////////////////////////////////////////////////////
std::string menu::getConfigurationFile()
{
	return strConfigurationFile;
}

/////////////////////////////////////////////////////////////////
bool menu::getbConfigurationFile()
{
	return bConfigurationFile;
}

unsigned short int menu::getNumberOfImages()
{
	return iNumberOfImages;
}

/////////////////////////////////////////////////////////////////
void menu::setKeyframePath(std::string keyframePath)
{
	vKeyframePath.push_back(keyframePath);
}

/////////////////////////////////////////////////////////////////
std::string menu::getKeyframePath(unsigned short int keyframeNumber)
{
	return vKeyframePath[keyframeNumber];
}

/////////////////////////////////////////////////////////////////
void menu::setInterpolationFrames(unsigned short int nInterpolationFrames)
{
	vInterpolationFrame.push_back(nInterpolationFrames);
}

/////////////////////////////////////////////////////////////////
unsigned short int menu::getInterpolationFrames(unsigned short int keyframeNumber)
{
	return vInterpolationFrame[keyframeNumber];
}

/////////////////////////////////////////////////////////////////
void menu::setMinDisplacement(float minHeightDisplacement)
{
	minDisplacement = minHeightDisplacement;
}

/////////////////////////////////////////////////////////////////
float menu::getMinDisplacement()
{
	return minDisplacement;
}

/////////////////////////////////////////////////////////////////
void menu::setMaxDisplacement(float maxHeightDisplacement)
{
	maxDisplacement = maxHeightDisplacement;
}

/////////////////////////////////////////////////////////////////
float menu::getMaxDisplacement()
{
	return maxDisplacement;
}

/////////////////////////////////////////////////////////////////
void menu::displayError()
{
	std::cout << "------------iBLENDER FILE ERROR------------" << std::endl;
	std::cout << "-------------------------------------------" << std::endl;
	std::cout << "Specified iBlender configuration file contains errors!" << std::endl;
	std::cout << std::endl;
	setConfigurationFile("");
	bConfigurationFile = false;
	showMainMenu();
}

/////////////////////////////////////////////////////////////////
void menu::showMainMenu()
{
	std::cout << "A. Specify an iBlender configuration file." << std::endl;

	if (bConfigurationFile == true)
	{
		std::cout << "B. View animation." << std::endl;
	}

	std::cout << "X. Exit iBlender." << std::endl;
	std::cout << std::endl;
	std::cout << "Menu item:";
	std::cin >> menuItem;

	switch(menuItem)
	{
		case 'a':
		case 'A':
			showMenuItem1();
			break;
		
		if (bConfigurationFile == true)
		{
			case 'b':
			case 'B':
				//showMenuItem2();
				break;
		}

		case 'x':
		case 'X':
			showMenuItem5();
			break;

		default:
			showErrorMessage();
			break;
	}
}

/////////////////////////////////////////////////////////////////
void menu::showMenuItem1()
{
	unsigned short int iTemp;
	float fTemp;
	std::string strTemp;

	std::cout << std::endl;
	std::cout << "             ##                                                " << std::endl;
	std::cout << "           ##  #                                               " << std::endl;
	std::cout << "         ##     #                                              " << std::endl;
	std::cout << "         #      #       #                        #             " << std::endl;
	std::cout << "    #    #      #       #                        #             " << std::endl;
	std::cout << "   ###  ##     #        #                        #             " << std::endl;
	std::cout << "    #   #     ##       ##                       ##             " << std::endl;
	std::cout << "        #     #        #                        #              " << std::endl;
	std::cout << "    #  ##    #  #      #                        #              " << std::endl;
	std::cout << "    #  #    # ## #    ##                       ##              " << std::endl;
	std::cout << "    #  # ###      #   #                        #               " << std::endl;
	std::cout << "   ## ###          #  #     ##    ####     #####   ##    # ### " << std::endl;
	std::cout << "   #  #            # ##    #  #  ##   #   #   ##  #  #  ###   #" << std::endl;
	std::cout << "   #  #            # #    #    # #    #  #    #  #    # #      " << std::endl;
	std::cout << "  ## ##            # #   #    #  #    # #     # #    #  #      " << std::endl;
	std::cout << "  #  #            # ##    # ##  ##    # #    ##  # ##  ##      " << std::endl;
	std::cout << "  #  #          ##  #      #    #    #   #   #    #    #       " << std::endl;
	std::cout << "  #  #        ##   #    #  #    #    #    #  #    #    #       " << std::endl;
	std::cout << " ### #########      ####    ### #    #     ####    ### #       " << std::endl;
	std::cout << std::endl;
	
	std::cout << "iBlender configuration file:";
	std::cin >> strTemp;
	setConfigurationFile(strTemp);

	//creates file stream and opens specified file for reading
	std::fstream finiBlender(getConfigurationFile().c_str(), std::ios::in);

	//if opening the file was successful
	if (finiBlender.good() == true)
	{
		finiBlender >> strTemp;
		bConfigurationFile = true;

		//check if it is a iBlender configuration file
		//if it is
		if (strTemp == "#iBLENDER_CONFIGURATION_FILE")
		{
			finiBlender >> strTemp;
			bConfigurationFile = true;

			if (strTemp == "#NUMBER_OF_KEYFRAMES")
			{
				finiBlender >> iNumberOfImages;
				mImage = new wrapper[iNumberOfImages];
				bConfigurationFile = true;

				finiBlender >> strTemp;

				if (strTemp == "#MINDISPLACEMENT")
				{
					finiBlender >> fTemp;
					setMinDisplacement(fTemp);
					bConfigurationFile = true;

					finiBlender >> strTemp;

					if (strTemp == "#MAXDISPLACEMENT")
					{
						finiBlender >> fTemp;
						setMaxDisplacement(fTemp);
						bConfigurationFile = true;

						while (finiBlender.eof() != true)
						{
							for (unsigned short int i = 0; i < getNumberOfImages(); i++)
							{
								finiBlender >> strTemp;

								if (strTemp == "#IMAGEPATH")
								{
									finiBlender >> strTemp;
									setKeyframePath(strTemp);
									mImage[i].setPath(getKeyframePath(i));
									bConfigurationFile = true;

									finiBlender >> strTemp;

									if (strTemp == "#NUMBER_OF_INTERPOLATION_FRAMES")
									{
										finiBlender >> iTemp;
										setInterpolationFrames(iTemp);
										bConfigurationFile = true;
									}
									else
									{
										displayError();
									}

									mImage[i].readImageInfo();
									mImage[i].numberOfPixels = static_cast <unsigned int>((mImage[i].getWidth() * mImage[i].getHeight()));
									mImage[i].createPixels();
								}
								else
								{
									displayError();
								}
							}
						}
					}
					else
					{
						displayError();
					}
				}
				else
				{
					displayError();
				}
			}
			else
			{
				displayError();
			}
			showMainMenu();
		}
		//if it isn't a iBlender configuration file
		else
		{
			std::cout << "------------iBLENDER FILE ERROR------------" << std::endl;
			std::cout << "-------------------------------------------" << std::endl;
			std::cout << "Specified file is not a iBlender configuration file!" << std::endl;
			std::cout << std::endl;
			setConfigurationFile("");
			bConfigurationFile = false;
			showMainMenu();
		}
	}
	//if the specified file doesn't exist or failed to open for some reason
	else
	{
		std::cout << "------------iBLENDER FILE ERROR------------" << std::endl;
		std::cout << "-------------------------------------------" << std::endl;
		std::cout << "Specified iBlender configuration file failed to open!" << std::endl;
		std::cout << std::endl;
		setConfigurationFile("");
		bConfigurationFile = false;
		showMainMenu();
	}
}

////////////////////////////////////////////////////////////////////////////////////
void menu::showMenuItem5()
{
	menuItem = ' ';
	std::cout << "Sure you want to quit? (Y / N)" << std::endl;
	std::cout << std::endl;
	std::cout << "Menu item:";
	std::cin >> menuItem;
	std::cout << std::endl;

	switch(menuItem)
	{
		case 'y':
		case 'Y':
			exit(0);
			break;

		case 'n':
		case 'N':
			showMainMenu();
			break;

		default:
			showErrorMessage();
			break;
	}
}

////////////////////////////////////////////////////////////////////////////////////
void menu::showErrorMessage()
{
	menuItem = ' ';
	std::cout << std::endl;
	std::cout << "Menu-item not defined." << std::endl;
	std::cout << std::endl;
	showMainMenu();
}