#define DO_OUTPUT 0
#pragma once
#include <iostream>
#include <fstream>
#include <time.h>
#include "wrapper.h"
#include "pixel.h"

////////////////////////////////////////////////////////////////////////////////////
//	default values for static members
//	ONLY CHANGE THESE LINES, DO NOT DELETE THEM!!
//	Static members must be initialized.
//	Since a header file can have more then 1 class definition in it,
//	we must provide the correct classname specification
vector wrapper::imageOffset (0, 0, 0);
vector wrapper::pixelSpacing (1, 1, 1);
vector wrapper::scatterVolumeScalar (2, 2, 2);
vector wrapper::pixelScalar (1, 1, 1);
vector wrapper::scatterVolumeDimension (1, 1, 1);

////////////////////////////////////////////////////////////////////////////////////
wrapper::wrapper()
{
	// Default constructor, 
	width = 0;
	height = 0;
	depth = 1;
	pixelDepth = 0;
	// ALWAYS INITIALIZE POINTERS TO NULL!!!
	pixels = NULL;
	colorMap = "";
}

////////////////////////////////////////////////////////////////////////////////////
//Constructor that takes a tga-object
wrapper::wrapper (LTGA tga)
{
	pixels = NULL;
	// Load the actual image
	loadImage(tga);
}

////////////////////////////////////////////////////////////////////////////////////
// We're using pointers so we seriously need a destructor to clean up
wrapper::~wrapper ()	
{
	// BEWARE: Before deleting an object (the object being pixels) we 
	//   need to be absolutly sure that the object does exist.
	if (pixels != NULL)
	{
		// Release the memory allocated to pixels
		delete [] pixels;
	}
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::setNumberOfPixels(unsigned int nPixels)
{
	numberOfPixels = nPixels;
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::setDepth(float iDepth)
{
	depth = iDepth;
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::loadImage (LTGA &tga)
{
	byte *iPixel;
	unsigned int pixelCount = 0;

	// getImageWidth() returns an unsigned int; our width is a float
	//   so a conversion is at the order.
	width = float(tga.getImageWidth());
	height = float(tga.getImageHeight());
	depth = 1;
	pixelDepth = tga.getPixelDepth();
	//+2 to give the pixels extra space, must the scatterVolumeScalar be set to (1, 1, 1).
	vector scatterVolumeDimension ((width * scatterVolumeScalar.x * pixelSpacing.x + 2), (height * scatterVolumeScalar.y * pixelSpacing.y + 2), (depth * scatterVolumeScalar.z * pixelSpacing.z + 2));
	//create pointer-array for pixels
	if (pixels != NULL)
	{
		delete [] pixels;
	}

	// Create the objects
	pixels = new pixel[getNumberOfPixels()];
	iPixel = tga.getPixels();

	if (getChannels() == 1)
	{
		// Instead of always defining 'i++' as the incremental part
		//   of the for-declaration, it's also a perfectly possible to 
		//   define the incremental part as a function or a complete calculation
		for (unsigned int i = 0; i < (getNumberOfPixels() * getChannels()); i = i + getChannels())
		{
			pixels[pixelCount].rgba.red = iPixel[i];
			pixels[pixelCount].rgba.green = iPixel[i];
			pixels[pixelCount].rgba.blue = iPixel[i];
			pixels[pixelCount].rgba.alpha = 255;
			pixelCount++;
		}
	}
	//RGB-RGBA image
	else if (getChannels() == 3 || getChannels() == 4)
	{
		for (unsigned int i = 0; i < (getNumberOfPixels() * getChannels()); i = i + getChannels())
		{
			pixels[pixelCount].rgba.red = iPixel[i];
			pixels[pixelCount].rgba.green = iPixel[i + 1];
			pixels[pixelCount].rgba.blue = iPixel[i + 2];
			pixels[pixelCount].rgba.alpha = 255;
			pixelCount++;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////////
bool wrapper::checkImage()
{
#if DO_OUTPUT == 1
	std::cout << "Entering checkImage procedure..." << std::endl;
#endif

	LTGA image;
	image.loadFromFile(getPath());

	if (image.isLoaded() == true)
	{
#if DO_OUTPUT == 1
		std::cout << "Specified image loaded successfully." << std::endl;
		std::cout << "Exiting checkImage procedure..." << std::endl;
		std::cout <<"----------------------------------------" << std::endl;
#endif
		return true;
	}
	else
	{
#if DO_OUTPUT == 1
		std::cout << "Specified image failed to load." << std::endl;
		std::cout << "Exiting checkImage procedure..." << std::endl;
		std::cout <<"----------------------------------------" << std::endl;
#endif
		return false;
	}

#if DO_OUTPUT == 1
	std::cout << "Exiting checkImage procedure..." << std::endl;
#endif
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::readImageInfo()
{
#if DO_OUTPUT == 1
	std::cout << "Entering readImageInfo procedure..." << std::endl;
#endif
	LTGA image;
	
	image.loadFromFile(getPath());

	// Load the image into the wrapper class
	loadImage(image);

	//print image info
	std::cout <<"----------------------------------------" << std::endl;
	std::cout << "Image:			" << getPath() << std::endl;
	std::cout << "Image size:		" << getWidth() << " x " << getHeight() << std::endl;
	std::cout << "# of pixels:		" << getNumberOfPixels() << std::endl;
	std::cout << "Pixel depth:		" << getPixelDepth() << std::endl;
	std::cout << "# of channels:\t	" << getChannels() << std::endl;
	std::cout <<"----------------------------------------" << std::endl;

#if DO_OUTPUT == 1
	std::cout << "Exiting readImageInfo procedure..." << std::endl;
	std::cout <<"----------------------------------------" << std::endl;
#endif
}

////////////////////////////////////////////////////////////////////////////////////
void wrapper::createPixels()
{
#if DO_OUTPUT == 1
	std::cout << "Entering createPixels procedure..." << std::endl;
#endif
	//create pixels
	unsigned int pixelCount = 0;

	for (int j = 0; j < getHeight(); j++)
	{
		for (int i = 0; i < getWidth(); i++)
		{
			vector vTemp;
			vTemp.x = ((((i - (getWidth() / 2.0f)) * pixelSpacing.x) + (pixelSpacing.x / 2.0f)) + imageOffset.x);
			vTemp.y = ((((j - (getHeight() / 2.0f)) * pixelSpacing.y) + (pixelSpacing.y / 2.0f)) + imageOffset.y);
			vTemp.z = imageOffset.z;
			pixels[pixelCount].setPosition (vTemp);
#if DO_OUTPUT == 1
			std::cout << vTemp << std::endl;
#endif
			pixelCount++;
		}
	}

#if DO_OUTPUT == 1
	std::cout << "Exiting createPixels procedure..." << std::endl;
	std::cout <<"----------------------------------------" << std::endl;
#endif
}